@@ -381,6 +381,8 @@ static int add_collation(struct charset_info_st *cs)
                                   &my_charset_utf8mb4_unicode_ci,
                                   cs);
         newcs->m_ctype= my_charset_utf8mb4_unicode_ci.m_ctype;
+        if (init_state_maps(newcs))
+          return MY_XML_ERROR;
         newcs->state|= MY_CS_AVAILABLE | MY_CS_LOADED;
 #endif
       }
@@ -667,13 +669,12 @@ const char *my_collation_get_tailoring(uint id)
 
 HASH charset_name_hash;
 
-static uchar *get_charset_key(const uchar *object,
-                              size_t *size,
-                              my_bool not_used __attribute__((unused)))
+static const uchar *get_charset_key(const void *object, size_t *size,
+                                    my_bool not_used __attribute__((unused)))
 {
-  CHARSET_INFO *cs= (CHARSET_INFO*) object;
+  CHARSET_INFO *cs= object;
   *size= strlen(cs->csname);
-  return (uchar*) cs->csname;
+  return (const uchar *) cs->csname;
 }
 
 static void init_available_charsets(void)
