@@ -18,6 +18,7 @@
 #include <mysys_priv.h>
 #include <my_default.h>
 #include <m_string.h>
+#include <ctype.h>
 #include <stdlib.h>
 #include <mysys_err.h>
 #include <my_getopt.h>
@@ -802,15 +803,21 @@ static int setval(const struct my_option *opts, void *value, char *argument,
       *((ulonglong*)value)= find_typeset(argument, opts->typelib, &err);
       if (err)
       {
-        /* Accept an integer representation of the set */
-        char *endptr;
-        ulonglong arg= (ulonglong) strtol(argument, &endptr, 10);
-        if (*endptr || (arg >> 1) >= (1ULL << (opts->typelib->count-1)))
+        /* Check if option 'all' is used (to set all bits) */
+        if (!my_strcasecmp(&my_charset_latin1, argument, "all"))
+          *(ulonglong*) value= ((1ULL << opts->typelib->count) - 1);
+        else
         {
-          res= EXIT_ARGUMENT_INVALID;
-          goto ret;
-        };
-        *(ulonglong*)value= arg;
+          /* Accept an integer representation of the set */
+          char *endptr;
+          ulonglong arg= (ulonglong) strtol(argument, &endptr, 10);
+          if (*endptr || (arg >> 1) >= (1ULL << (opts->typelib->count-1)))
+          {
+            res= EXIT_ARGUMENT_INVALID;
+            goto ret;
+          };
+          *(ulonglong*)value= arg;
+        }
         err= 0;
       }
       break;
@@ -956,7 +963,7 @@ my_bool getopt_compare_strings(register const char *s, register const char *t,
 
   for (;s != end ; s++, t++)
   {
-    if ((*s != '-' ? *s : '_') != (*t != '-' ? *t : '_'))
+    if ((*s != '-' ? tolower(*s) : '_') != (*t != '-' ? tolower(*t) : '_'))
       DBUG_RETURN(1);
   }
   DBUG_RETURN(0);
