@@ -67,8 +67,8 @@ protected:
     memcpy(m_buffer, str, length);
     return false;
   }
-  // Non-initializing constructor
-  Inet4() { }
+
+  Inet4() = default;
 public:
   void to_binary(char *dst, size_t dstsize) const
   {
@@ -161,7 +161,7 @@ protected:
     return false;
   }
 
-  Inet6() { }
+  Inet6() = default;
 
 public:
   static uint binary_length() { return IN6_ADDR_SIZE; }
@@ -188,6 +188,21 @@ public:
   */
   static bool fix_fields_maybe_null_on_conversion_to_inet6(Item *item);
 
+  /*
+    Check at fix_fields() time if any of the items can return a nullable
+    value on conversion to Fbt.
+  */
+  static bool fix_fields_maybe_null_on_conversion_to_inet6(Item **items,
+                                                           uint count)
+  {
+    for (uint i= 0; i < count; i++)
+    {
+      if (fix_fields_maybe_null_on_conversion_to_inet6(items[i]))
+        return true;
+    }
+    return false;
+  }
+
 public:
 
   Inet6(Item *item, bool *error, bool warn= true)
@@ -528,11 +543,11 @@ public:
   }
   void make_sort_key_part(uchar *to, Item *item,
                           const SORT_FIELD_ATTR *sort_field,
-                          Sort_param *param)
+                          String *tmp)
                      const override;
   uint make_packed_sort_key_part(uchar *to, Item *item,
                                  const SORT_FIELD_ATTR *sort_field,
-                                 Sort_param *param) const override;
+                                 String *tmp) const override;
   void sort_length(THD *thd,
                   const Type_std_attributes *item,
                   SORT_FIELD_ATTR *attr) const override;
@@ -714,6 +729,16 @@ public:
     Inet6_null nb(b);
     return !na.is_null() && !nb.is_null() && !na.cmp(nb);
   }
+  bool Item_bool_rowready_func2_fix_length_and_dec(THD *thd,
+                                 Item_bool_rowready_func2 *func) const override
+  {
+    if (Type_handler::Item_bool_rowready_func2_fix_length_and_dec(thd, func))
+      return true;
+    if (!func->maybe_null &&
+        Inet6::fix_fields_maybe_null_on_conversion_to_inet6(func->arguments(), 2))
+      func->maybe_null= true;
+    return false;
+  }
   bool Item_hybrid_func_fix_attributes(THD *thd,
                                        const char *name,
                                        Type_handler_hybrid_field_type *h,
@@ -902,6 +927,10 @@ public:
   bool
   Item_func_between_fix_length_and_dec(Item_func_between *func) const override
   {
+    if (!func->maybe_null &&
+        Inet6::fix_fields_maybe_null_on_conversion_to_inet6(func->arguments(), 3))
+      func->maybe_null= true;
+
     return false;
   }
   longlong Item_func_between_val_int(Item_func_between *func) const override
@@ -918,6 +947,10 @@ public:
                                                     Item_func_in *func)
                                                     const override
   {
+    if (!func->maybe_null &&
+        Inet6::fix_fields_maybe_null_on_conversion_to_inet6(func->arguments(),
+                                                            func->argument_count()))
+      func->maybe_null= true;
     if (func->compatible_types_scalar_bisection_possible())
     {
       return func->value_list_convert_const_to_int(thd) ||
