@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
 
 #define MYSQL_LEX 1
 #include "mariadb.h"                          /* NO_EMBEDDED_ACCESS_CHECKS */
@@ -25,19 +25,34 @@
                                            // date_add_interval,
                                            // calc_time_diff
 #include "tztime.h"     // my_tz_find, my_tz_OFFSET0, struct Time_zone
-#include "sql_acl.h"    // EVENT_ACL, SUPER_ACL
 #include "sp.h"         // load_charset, load_collation
 #include "events.h"
 #include "event_data_objects.h"
 #include "event_db_repository.h"
 #include "sp_head.h"
 #include "sql_show.h"                // append_definer, append_identifier
-
+#include "mysql/psi/mysql_sp.h"
+#include "wsrep_mysqld.h"
+#ifdef WITH_WSREP
+#include "wsrep_trans_observer.h"
+#endif /* WITH_WSREP */
 /**
   @addtogroup Event_Scheduler
   @{
 */
 
+#ifdef HAVE_PSI_INTERFACE
+void init_scheduler_psi_keys()
+{
+  const char *category= "scheduler";
+
+  PSI_server->register_statement(category, & Event_queue_element_for_exec::psi_info, 1);
+}
+
+PSI_statement_info Event_queue_element_for_exec::psi_info=
+{ 0, "event", 0};
+#endif
+
 /*************************************************************************/
 
 /**
@@ -56,14 +71,14 @@ public:
                            Stored_program_creation_ctx **ctx);
 
 public:
-  virtual Stored_program_creation_ctx *clone(MEM_ROOT *mem_root)
+  Stored_program_creation_ctx *clone(MEM_ROOT *mem_root) override
   {
     return new (mem_root)
                Event_creation_ctx(m_client_cs, m_connection_cl, m_db_cl);
   }
 
 protected:
-  virtual Object_creation_ctx *create_backup_ctx(THD *thd) const
+  Object_creation_ctx *create_backup_ctx(THD *thd) const override
   {
     /*
       We can avoid usual backup/restore employed in stored programs since we
@@ -159,7 +174,7 @@ Event_creation_ctx::load_from_db(THD *thd,
 /*************************************************************************/
 
 /*
-  Initiliazes dbname and name of an Event_queue_element_for_exec
+  Initializes dbname and name of an Event_queue_element_for_exec
   object
 
   SYNOPSIS
@@ -171,13 +186,16 @@ Event_creation_ctx::load_from_db(THD *thd,
 */
 
 bool
-Event_queue_element_for_exec::init(const LEX_CSTRING *db, const LEX_CSTRING *n)
+Event_queue_element_for_exec::init(const LEX_CSTRING &db, const LEX_CSTRING &n)
 {
-  if (!(dbname.str= my_strndup(db->str, dbname.length= db->length, MYF(MY_WME))))
+  if (!(dbname.str= my_strndup(key_memory_Event_queue_element_for_exec_names,
+                               db.str, dbname.length= db.length, MYF(MY_WME))))
     return TRUE;
-  if (!(name.str= my_strndup(n->str, name.length= n->length, MYF(MY_WME))))
+  if (!(name.str= my_strndup(key_memory_Event_queue_element_for_exec_names,
+                             n.str, name.length= n.length, MYF(MY_WME))))
   {
     my_free(const_cast<char*>(dbname.str));
+    dbname.str= NULL;
     return TRUE;
   }
   return FALSE;
@@ -209,7 +227,7 @@ Event_basic::Event_basic()
 {
   DBUG_ENTER("Event_basic::Event_basic");
   /* init memory root */
-  init_sql_alloc(&mem_root, "Event_basic", 256, 512, MYF(0));
+  init_sql_alloc(key_memory_event_basic_root, &mem_root, 256, 512, MYF(0));
   dbname.str= name.str= NULL;
   dbname.length= name.length= 0;
   time_zone= NULL;
@@ -310,9 +328,7 @@ Event_queue_element::Event_queue_element():
   SYNOPSIS
     Event_queue_element::Event_queue_element()
 */
-Event_queue_element::~Event_queue_element()
-{
-}
+Event_queue_element::~Event_queue_element() = default;
 
 
 /*
@@ -338,9 +354,7 @@ Event_timed::Event_timed():
     Event_timed::~Event_timed()
 */
 
-Event_timed::~Event_timed()
-{
-}
+Event_timed::~Event_timed() = default;
 
 
 /*
@@ -479,14 +493,24 @@ Event_queue_element::load_from_row(THD *thd, TABLE *table)
   uint not_used;
   if (!starts_null)
   {
-    table->field[ET_FIELD_STARTS]->get_date(&time, TIME_NO_ZERO_DATE);
+    /*
+      The expected data type for these columns in mysql.events:
+        starts, ends, execute_at, last_executed
+      is DATETIME. No nanosecond truncation should normally be needed,
+      unless the DBA changes them, e.g. to VARCHAR, DECIMAL, etc.
+      For this unexpected case let's use the default round mode,
+      according to the current session settings.
+    */
+    table->field[ET_FIELD_STARTS]->get_date(&time, TIME_NO_ZERO_DATE |
+                                                   thd->temporal_round_mode());
     starts= my_tz_OFFSET0->TIME_to_gmt_sec(&time,&not_used);
   }
 
   ends_null= table->field[ET_FIELD_ENDS]->is_null();
   if (!ends_null)
   {
-    table->field[ET_FIELD_ENDS]->get_date(&time, TIME_NO_ZERO_DATE);
+    table->field[ET_FIELD_ENDS]->get_date(&time, TIME_NO_ZERO_DATE |
+                                                 thd->temporal_round_mode());
     ends= my_tz_OFFSET0->TIME_to_gmt_sec(&time,&not_used);
   }
 
@@ -502,8 +526,8 @@ Event_queue_element::load_from_row(THD *thd, TABLE *table)
   DBUG_ASSERT(!(starts_null && ends_null && !expression && execute_at_null));
   if (!expression && !execute_at_null)
   {
-    if (table->field[ET_FIELD_EXECUTE_AT]->get_date(&time,
-                                                    TIME_NO_ZERO_DATE))
+    if (table->field[ET_FIELD_EXECUTE_AT]->get_date(&time, TIME_NO_ZERO_DATE |
+                                                    thd->temporal_round_mode()))
       DBUG_RETURN(TRUE);
     execute_at= my_tz_OFFSET0->TIME_to_gmt_sec(&time,&not_used);
   }
@@ -535,8 +559,8 @@ Event_queue_element::load_from_row(THD *thd, TABLE *table)
 
   if (!table->field[ET_FIELD_LAST_EXECUTED]->is_null())
   {
-    table->field[ET_FIELD_LAST_EXECUTED]->get_date(&time,
-                                                   TIME_NO_ZERO_DATE);
+    table->field[ET_FIELD_LAST_EXECUTED]->get_date(&time, TIME_NO_ZERO_DATE |
+                                                   thd->temporal_round_mode());
     last_executed= my_tz_OFFSET0->TIME_to_gmt_sec(&time,&not_used);
   }
 
@@ -654,7 +678,7 @@ my_time_t
 add_interval(MYSQL_TIME *ltime, const Time_zone *time_zone,
              interval_type scale, INTERVAL interval)
 {
-  if (date_add_interval(ltime, scale, interval))
+  if (date_add_interval(current_thd, ltime, scale, interval))
     return 0;
 
   uint not_used;
@@ -758,8 +782,8 @@ bool get_next_time(const Time_zone *time_zone, my_time_t *next,
 
   if (seconds)
   {
-    longlong seconds_diff;
-    long microsec_diff;
+    ulonglong seconds_diff;
+    ulong microsec_diff;
     bool negative= calc_time_diff(&local_now, &local_start, 1,
                                   &seconds_diff, &microsec_diff);
     if (!negative)
@@ -1343,6 +1367,10 @@ Event_job_data::execute(THD *thd, bool drop)
 
   thd->reset_for_next_command();
 
+#ifdef WITH_WSREP
+  wsrep_open(thd);
+  wsrep_before_command(thd);
+#endif /* WITH_WSREP */
   /*
     MySQL parser currently assumes that current database is either
     present in THD or all names in all statements are fully specified.
@@ -1409,15 +1437,23 @@ Event_job_data::execute(THD *thd, bool drop)
 
   {
     Parser_state parser_state;
+    sql_digest_state *parent_digest= thd->m_digest;
+    PSI_statement_locker *parent_locker= thd->m_statement_psi;
+    bool res;
+
     if (parser_state.init(thd, thd->query(), thd->query_length()))
       goto end;
 
-    if (parse_sql(thd, & parser_state, creation_ctx))
+    thd->m_digest= NULL;
+    thd->m_statement_psi= NULL;
+    res= parse_sql(thd, & parser_state, creation_ctx);
+    thd->m_digest= parent_digest;
+    thd->m_statement_psi= parent_locker;
+
+    if (res)
     {
-      sql_print_error("Event Scheduler: "
-                      "%serror during compilation of %s.%s",
-                      thd->is_fatal_error ? "fatal " : "",
-                      (const char *) dbname.str, (const char *) name.str);
+      sql_print_error("Event Scheduler: %serror during compilation of %s.%s",
+                      thd->is_fatal_error ? "fatal " : "", dbname.str, name.str);
       goto end;
     }
   }
@@ -1440,6 +1476,9 @@ Event_job_data::execute(THD *thd, bool drop)
     sphead->set_creation_ctx(creation_ctx);
     sphead->optimize();
 
+    sphead->m_sp_share= MYSQL_GET_SP_SHARE(SP_TYPE_EVENT,
+                                           dbname.str, static_cast<uint>(dbname.length),
+                                           name.str, static_cast<uint>(name.length));
     ret= sphead->execute_procedure(thd, &empty_item_list);
     /*
       There is no pre-locking and therefore there should be no
@@ -1464,8 +1503,6 @@ end:
       ret= 1;
     else
     {
-      ulong saved_master_access;
-
       thd->set_query(sp_sql.c_ptr_safe(), sp_sql.length());
 
       /*
@@ -1477,8 +1514,8 @@ end:
         Temporarily reset it to read-write.
       */
 
-      saved_master_access= thd->security_ctx->master_access;
-      thd->security_ctx->master_access |= SUPER_ACL;
+      privilege_t saved_master_access(thd->security_ctx->master_access);
+      thd->security_ctx->master_access |= PRIV_IGNORE_READ_ONLY;
       bool save_tx_read_only= thd->tx_read_only;
       thd->tx_read_only= false;
 
@@ -1517,6 +1554,10 @@ end:
   if (save_sctx)
     event_sctx.restore_security_context(thd, save_sctx);
 #endif
+#ifdef WITH_WSREP
+  wsrep_after_command_ignore_result(thd);
+  wsrep_close(thd);
+#endif /* WITH_WSREP */
   thd->lex->unit.cleanup();
   thd->end_statement();
   thd->cleanup_after_query();
