@@ -141,21 +141,21 @@ public:
     fltend= (MY_XPATH_FLT*) tmp_native_value.end();
     nodeset->length(0);
   }
-  const Type_handler *type_handler() const
+  const Type_handler *type_handler() const override
   {
     return &type_handler_xpath_nodeset;
   }
-  const Type_handler *fixed_type_handler() const
+  const Type_handler *fixed_type_handler() const override
   {
     return &type_handler_xpath_nodeset;
   }
   Field *create_tmp_field_ex(MEM_ROOT *root, TABLE *table, Tmp_field_src *src,
-                             const Tmp_field_param *param)
+                             const Tmp_field_param *param) override
   {
     DBUG_ASSERT(0);
     return NULL;
   }
-  String *val_str(String *str)
+  String *val_str(String *str) override
   {
     prepare_nodes();
     val_native(current_thd, &tmp2_native_value);
@@ -189,7 +189,7 @@ public:
     }
     return str;
   }
-  bool fix_length_and_dec()
+  bool fix_length_and_dec() override
   {
     max_length= MAX_BLOB_WIDTH;
     collation.collation= pxml->charset();
@@ -198,8 +198,8 @@ public:
     const_item_cache= false;
     return FALSE;
   }
-  const char *func_name() const { return "nodeset"; }
-  bool check_vcol_func_processor(void *arg)
+  const char *func_name() const override { return "nodeset"; }
+  bool check_vcol_func_processor(void *arg) override
   {
     return mark_unsupported_function(func_name(), arg, VCOL_IMPOSSIBLE);
   }
@@ -213,9 +213,9 @@ class Item_nodeset_func_rootelement :public Item_nodeset_func
 public:
   Item_nodeset_func_rootelement(THD *thd, String *pxml):
     Item_nodeset_func(thd, pxml) {}
-  const char *func_name() const { return "xpath_rootelement"; }
-  bool val_native(THD *thd, Native *nodeset);
-  Item *get_copy(THD *thd)
+  const char *func_name() const override { return "xpath_rootelement"; }
+  bool val_native(THD *thd, Native *nodeset) override;
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_rootelement>(thd, this); }
 };
 
@@ -226,9 +226,9 @@ class Item_nodeset_func_union :public Item_nodeset_func
 public:
   Item_nodeset_func_union(THD *thd, Item *a, Item *b, String *pxml):
     Item_nodeset_func(thd, a, b, pxml) {}
-  const char *func_name() const { return "xpath_union"; }
-  bool val_native(THD *thd, Native *nodeset);
-  Item *get_copy(THD *thd)
+  const char *func_name() const override { return "xpath_union"; }
+  bool val_native(THD *thd, Native *nodeset) override;
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_union>(thd, this); }
 };
 
@@ -242,7 +242,7 @@ public:
   Item_nodeset_func_axisbyname(THD *thd, Item *a, const char *n_arg, uint l_arg,
                                String *pxml):
     Item_nodeset_func(thd, a, pxml), node_name(n_arg), node_namelen(l_arg) { }
-  const char *func_name() const { return "xpath_axisbyname"; }
+  const char *func_name() const override { return "xpath_axisbyname"; }
   bool validname(MY_XML_NODE *n)
   {
     if (node_name[0] == '*')
@@ -260,9 +260,9 @@ public:
   Item_nodeset_func_selfbyname(THD *thd, Item *a, const char *n_arg, uint l_arg,
                                String *pxml):
     Item_nodeset_func_axisbyname(thd, a, n_arg, l_arg, pxml) {}
-  const char *func_name() const { return "xpath_selfbyname"; }
-  bool val_native(THD *thd, Native *nodeset);
-  Item *get_copy(THD *thd)
+  const char *func_name() const override { return "xpath_selfbyname"; }
+  bool val_native(THD *thd, Native *nodeset) override;
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_selfbyname>(thd, this); }
 };
 
@@ -274,9 +274,9 @@ public:
   Item_nodeset_func_childbyname(THD *thd, Item *a, const char *n_arg, uint l_arg,
                                 String *pxml): 
     Item_nodeset_func_axisbyname(thd, a, n_arg, l_arg, pxml) {}
-  const char *func_name() const { return "xpath_childbyname"; }
-  bool val_native(THD *thd, Native *nodeset);
-  Item *get_copy(THD *thd)
+  const char *func_name() const override { return "xpath_childbyname"; }
+  bool val_native(THD *thd, Native *nodeset) override;
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_childbyname>(thd, this); }
 };
 
@@ -290,9 +290,9 @@ public:
                                      String *pxml, bool need_self_arg):
     Item_nodeset_func_axisbyname(thd, a, n_arg, l_arg, pxml),
       need_self(need_self_arg) {}
-  const char *func_name() const { return "xpath_descendantbyname"; }
-  bool val_native(THD *thd, Native *nodeset);
-  Item *get_copy(THD *thd)
+  const char *func_name() const override { return "xpath_descendantbyname"; }
+  bool val_native(THD *thd, Native *nodeset) override;
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_descendantbyname>(thd, this); }
 };
 
@@ -306,9 +306,9 @@ public:
                                    String *pxml, bool need_self_arg):
     Item_nodeset_func_axisbyname(thd, a, n_arg, l_arg, pxml),
       need_self(need_self_arg) {}
-  const char *func_name() const { return "xpath_ancestorbyname"; }
-  bool val_native(THD *thd, Native *nodeset);
-  Item *get_copy(THD *thd)
+  const char *func_name() const override { return "xpath_ancestorbyname"; }
+  bool val_native(THD *thd, Native *nodeset) override;
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_ancestorbyname>(thd, this); }
 };
 
@@ -320,9 +320,9 @@ public:
   Item_nodeset_func_parentbyname(THD *thd, Item *a, const char *n_arg, uint l_arg,
                                  String *pxml):
     Item_nodeset_func_axisbyname(thd, a, n_arg, l_arg, pxml) {}
-  const char *func_name() const { return "xpath_parentbyname"; }
-  bool val_native(THD *thd, Native *nodeset);
-  Item *get_copy(THD *thd)
+  const char *func_name() const override { return "xpath_parentbyname"; }
+  bool val_native(THD *thd, Native *nodeset) override;
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_parentbyname>(thd, this); }
 };
 
@@ -334,9 +334,9 @@ public:
   Item_nodeset_func_attributebyname(THD *thd, Item *a, const char *n_arg,
                                     uint l_arg, String *pxml):
     Item_nodeset_func_axisbyname(thd, a, n_arg, l_arg, pxml) {}
-  const char *func_name() const { return "xpath_attributebyname"; }
-  bool val_native(THD *thd, Native *nodeset);
-  Item *get_copy(THD *thd)
+  const char *func_name() const override { return "xpath_attributebyname"; }
+  bool val_native(THD *thd, Native *nodeset) override;
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_attributebyname>(thd, this); }
 };
 
@@ -351,9 +351,9 @@ class Item_nodeset_func_predicate :public Item_nodeset_func
 public:
   Item_nodeset_func_predicate(THD *thd, Item *a, Item *b, String *pxml):
     Item_nodeset_func(thd, a, b, pxml) {}
-  const char *func_name() const { return "xpath_predicate"; }
-  bool val_native(THD *thd, Native *nodeset);
-  Item *get_copy(THD *thd)
+  const char *func_name() const override { return "xpath_predicate"; }
+  bool val_native(THD *thd, Native *nodeset) override;
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_predicate>(thd, this); }
 };
 
@@ -364,9 +364,9 @@ class Item_nodeset_func_elementbyindex :public Item_nodeset_func
 public:
   Item_nodeset_func_elementbyindex(THD *thd, Item *a, Item *b, String *pxml):
     Item_nodeset_func(thd, a, b, pxml) { }
-  const char *func_name() const { return "xpath_elementbyindex"; }
-  bool val_native(THD *thd, Native *nodeset);
-  Item *get_copy(THD *thd)
+  const char *func_name() const override { return "xpath_elementbyindex"; }
+  bool val_native(THD *thd, Native *nodeset) override;
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_elementbyindex>(thd, this); }
 };
 
@@ -384,8 +384,8 @@ class Item_xpath_cast_bool :public Item_bool_func
 public:
   Item_xpath_cast_bool(THD *thd, Item *a, String *pxml_arg):
     Item_bool_func(thd, a), pxml(pxml_arg) {}
-  const char *func_name() const { return "xpath_cast_bool"; }
-  longlong val_int()
+  const char *func_name() const override { return "xpath_cast_bool"; }
+  longlong val_int() override
   {
     if (args[0]->fixed_type_handler() == &type_handler_xpath_nodeset)
     {
@@ -394,7 +394,7 @@ public:
     }
     return args[0]->val_real() ? 1 : 0;
   }
-  Item *get_copy(THD *thd)
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_xpath_cast_bool>(thd, this); }
 };
 
@@ -406,9 +406,9 @@ class Item_xpath_cast_number :public Item_real_func
 {
 public:
   Item_xpath_cast_number(THD *thd, Item *a): Item_real_func(thd, a) {}
-  const char *func_name() const { return "xpath_cast_number"; }
-  virtual double val_real() { return args[0]->val_real(); }
-  Item *get_copy(THD *thd)
+  const char *func_name() const override { return "xpath_cast_number"; }
+  double val_real() override { return args[0]->val_real(); }
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_xpath_cast_number>(thd, this); }
 };
 
@@ -422,12 +422,12 @@ public:
   Native *native_cache;
   Item_nodeset_context_cache(THD *thd, Native *native_arg, String *pxml):
     Item_nodeset_func(thd, pxml), native_cache(native_arg) { }
-  bool val_native(THD *thd, Native *nodeset)
+  bool val_native(THD *thd, Native *nodeset) override
   {
     return nodeset->copy(*native_cache);
   }
-  bool fix_length_and_dec() { max_length= MAX_BLOB_WIDTH;; return FALSE;  }
-  Item *get_copy(THD *thd)
+  bool fix_length_and_dec() override { max_length= MAX_BLOB_WIDTH;; return FALSE;  }
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_context_cache>(thd, this); }
 };
 
@@ -439,16 +439,16 @@ class Item_func_xpath_position :public Item_long_func
 public:
   Item_func_xpath_position(THD *thd, Item *a, String *p):
     Item_long_func(thd, a), pxml(p) {}
-  const char *func_name() const { return "xpath_position"; }
-  bool fix_length_and_dec() { max_length=10; return FALSE; }
-  longlong val_int()
+  const char *func_name() const override { return "xpath_position"; }
+  bool fix_length_and_dec() override { max_length=10; return FALSE; }
+  longlong val_int() override
   {
     args[0]->val_native(current_thd, &tmp_native_value);
     if (tmp_native_value.elements() == 1)
       return tmp_native_value.element(0).pos + 1;
     return 0;
   }
-  Item *get_copy(THD *thd)
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_xpath_position>(thd, this); }
 };
 
@@ -460,9 +460,9 @@ class Item_func_xpath_count :public Item_long_func
 public:
   Item_func_xpath_count(THD *thd, Item *a, String *p):
     Item_long_func(thd, a), pxml(p) {}
-  const char *func_name() const { return "xpath_count"; }
-  bool fix_length_and_dec() { max_length=10; return FALSE; }
-  longlong val_int()
+  const char *func_name() const override { return "xpath_count"; }
+  bool fix_length_and_dec() override { max_length=10; return FALSE; }
+  longlong val_int() override
   {
     uint predicate_supplied_context_size;
     args[0]->val_native(current_thd, &tmp_native_value);
@@ -471,7 +471,7 @@ public:
       return predicate_supplied_context_size;
     return tmp_native_value.elements();
   }
-  Item *get_copy(THD *thd)
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_xpath_count>(thd, this); }
 };
 
@@ -484,8 +484,8 @@ public:
   Item_func_xpath_sum(THD *thd, Item *a, String *p):
     Item_real_func(thd, a), pxml(p) {}
 
-  const char *func_name() const { return "xpath_sum"; }
-  double val_real()
+  const char *func_name() const override { return "xpath_sum"; }
+  double val_real() override
   {
     double sum= 0;
     args[0]->val_native(current_thd, &tmp_native_value);
@@ -516,7 +516,7 @@ public:
     }
     return sum;
   }
-  Item *get_copy(THD *thd)
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_xpath_sum>(thd, this); }
 };
 
@@ -531,13 +531,13 @@ public:
                             CHARSET_INFO *cs):
     Item_string(thd, str, length, cs)
   { }
-  bool const_item() const { return false ; }
-  bool basic_const_item() const { return false; }
+  bool const_item() const override { return false ; }
+  bool basic_const_item() const override { return false; }
   void set_value(const char *str, uint length, CHARSET_INFO *cs)
   {
     str_value.set(str, length, cs);
   }
-  Item *safe_charset_converter(THD *thd, CHARSET_INFO *tocs)
+  Item *safe_charset_converter(THD *thd, CHARSET_INFO *tocs) override
   {
     /*
       Item_string::safe_charset_converter() does not accept non-constants.
@@ -556,18 +556,18 @@ public:
   Item_nodeset_to_const_comparator(THD *thd, Item *nodeset, Item *cmpfunc,
                                    String *p):
     Item_bool_func(thd, nodeset, cmpfunc), pxml(p) {}
-  const char *func_name() const { return "xpath_nodeset_to_const_comparator"; }
-  bool check_vcol_func_processor(void *arg)
+  const char *func_name() const override { return "xpath_nodeset_to_const_comparator"; }
+  bool check_vcol_func_processor(void *arg) override
   {
     return mark_unsupported_function(func_name(), arg, VCOL_IMPOSSIBLE);
   }
   Field *create_tmp_field_ex(MEM_ROOT *root, TABLE *table, Tmp_field_src *src,
-                             const Tmp_field_param *param)
+                             const Tmp_field_param *param) override
   {
     DBUG_ASSERT(0);
     return NULL;
   }
-  longlong val_int()
+  longlong val_int() override
   {
     Item_func *comp= (Item_func*)args[1];
     Item_string_xml_non_const *fake=
@@ -598,7 +598,7 @@ public:
     }
     return 0;
   }
-  Item *get_copy(THD *thd)
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_to_const_comparator>(thd, this); }
 };
 
@@ -2591,19 +2591,21 @@ static int
 my_xpath_parse_VariableReference(MY_XPATH *xpath)
 {
   LEX_CSTRING name;
-  int user_var;
-  const char *dollar_pos;
   THD *thd= xpath->thd;
-  if (!my_xpath_parse_term(xpath, MY_XPATH_LEX_DOLLAR) ||
-      (!(dollar_pos= xpath->prevtok.beg)) ||
-      (!((user_var= my_xpath_parse_term(xpath, MY_XPATH_LEX_AT) &&
+  if (!my_xpath_parse_term(xpath, MY_XPATH_LEX_DOLLAR))
+    return 0;
+  const char *dollar_pos= xpath->prevtok.beg;
+  if (!dollar_pos)
+    return 0;
+  int user_var= my_xpath_parse_term(xpath, MY_XPATH_LEX_AT);
+  if (!((user_var &&
          my_xpath_parse_term(xpath, MY_XPATH_LEX_IDENT))) &&
-       !my_xpath_parse_term(xpath, MY_XPATH_LEX_IDENT)))
+      !my_xpath_parse_term(xpath, MY_XPATH_LEX_IDENT))
     return 0;
 
   name.length= xpath->prevtok.end - xpath->prevtok.beg;
   name.str= (char*) xpath->prevtok.beg;
-  
+
   if (user_var)
     xpath->item= new (thd->mem_root) Item_func_get_user_var(thd, &name);
   else
