@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2017 MariaDB
+   Copyright (c) 2017, 2020, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -65,7 +65,7 @@
   If we have only one equi-join condition then we either push it as
   for Q1R or we don't. In a general case we may have much more options.
   Consider the query (Q3)
-    SELECT
+    SELECT *
       FROM t1,t2 (SELECT t3.a, t3.b, MIN(t3.c) as min
                   FROM t3 GROUP BY a,b) t
     WHERE t.a = t1.a AND t.b = t2.b
@@ -102,6 +102,47 @@
   If we just drop the index on t3(a,b) the chances that the splitting
   will be used becomes much lower but they still exists providing that
   the fanout of the partial join of t1 and t2 is small enough.
+
+  The lateral derived table LT formed as a result of SM optimization applied
+  to a materialized derived table DT must be joined after all parameters
+  of splitting has been evaluated, i.e. after all expressions used in the
+  equalities pushed into DT that make the employed splitting effective
+  could be evaluated. With the chosen join order all the parameters can be
+  evaluated after the last table LPT that contains any columns referenced in
+  the parameters has been joined and the table APT following LPT in the chosen
+  join order is accessed.
+  Usually the formed lateral derived table LT is accessed right after the table
+  LPT. As in such cases table LT must be refilled for each combination of
+  splitting parameters this table must be populated before each access to LT
+  and the estimate of the expected number of refills that could be suggested in
+  such cases is the number of rows in the partial join ending with table LPT.
+  However in other cases the chosen join order may contain tables between LPT
+  and LT.
+  Consider the query (Q4)
+    SELECT *
+      FROM t1 JOIN t2 ON t1.b = t2.b
+           LEFT JOIN  (SELECT t3.a, t3.b, MIN(t3.c) as min
+                       FROM t3 GROUP BY a,b) t
+           ON t.a = t1.a AND t.c > 0
+      [WHERE P(t1,t2)];
+  Let's assume that the join order t1,t2,t was chosen for this query and
+  SP optimization was applied to t with splitting over t3.a using the index
+  on column t3.a. Here the table t1 serves as LPT, t2 as APT while t with
+  pushed condition t.a = t1.a serves as LT. Note that here LT is accessed
+  after t2,  not right after t1. Here the number of refills of the lateral
+  derived is not more that the  number of key values of t1.a that might be
+  less than the cardinality of the partial join (t1,t2). That's why it makes
+  sense to signal that t3 has to be refilled just before t2 is accessed.
+  However if the cardinality of the partial join (t1,t2) happens to be less
+  than the cardinality of the partial join (t1) due to additional selective
+  condition P(t1,t2) then the flag informing about necessity of a new refill
+  can be set either when accessing t2 or right after it has been joined.
+  The current code sets such flag right after generating a record of the
+  partial join with minimal cardinality for all those partial joins that
+  end between APT and LT. It allows sometimes to push extra conditions
+  into the lateral derived without any increase of the number of refills.
+  However this flag can be set only after the last join table between
+  APT and LT using join buffer has been joined.
 */
 
 /*
@@ -187,6 +228,7 @@
 
 #include "mariadb.h"
 #include "sql_select.h"
+#include "opt_trace.h"
 
 /* Info on a splitting field */
 struct SplM_field_info
@@ -204,7 +246,7 @@ struct SplM_field_info
 struct SplM_plan_info
 {
   /* The cached splitting execution plan P */
-  struct st_position *best_positions;
+  POSITION *best_positions;
   /* The cost of the above plan */
   double cost;
   /* Selectivity of splitting used in P */
@@ -236,6 +278,8 @@ public:
   SplM_field_info *spl_fields;
   /* The number of elements in the above list */
   uint spl_field_cnt;
+  /* The list of equalities injected into WHERE for split optimization */
+  List<Item> inj_cond_list;
   /* Contains the structures to generate all KEYUSEs for pushable equalities */
   List<KEY_FIELD> added_key_fields;
   /* The cache of evaluated execution plans for 'join' with pushed equalities */
@@ -246,6 +290,7 @@ public:
   double unsplit_card;
   /* Lastly evaluated execution plan for 'join' with pushed equalities */
   SplM_plan_info *last_plan;
+  double last_refills;
 
   SplM_plan_info *find_plan(TABLE *table, uint key, uint parts);
 };
@@ -308,6 +353,8 @@ struct SplM_field_ext_info: public SplM_field_info
        occurred also in the select list of this join
     9. There are defined some keys usable for ref access of fields from C
        with available statistics.
+    10. The select doesn't use WITH ROLLUP (This limitation can probably be
+       lifted)
 
   @retval
     true   if the answer is positive
@@ -324,7 +371,8 @@ bool JOIN::check_for_splittable_materialized()
       (unit->first_select()->next_select()) ||                        // !(3)
       (derived->prohibit_cond_pushdown) ||                            // !(4)
       (derived->is_recursive_with_table()) ||                         // !(5)
-      (table_count == 0 || const_tables == top_join_tab_count))       // !(6)
+      (table_count == 0 || const_tables == top_join_tab_count) ||     // !(6)
+      rollup.state != ROLLUP::STATE_NONE)                             // (10)
     return false;
   if (group_list)                                                     // (7.1)
   {
@@ -340,6 +388,9 @@ bool JOIN::check_for_splittable_materialized()
   if (!partition_list)
     return false;
 
+  Json_writer_object trace_wrapper(thd);
+  Json_writer_object trace_split(thd, "check_split_materialized");
+
   ORDER *ord;
   Dynamic_array<SplM_field_ext_info> candidates(PSI_INSTRUMENT_MEM);
 
@@ -385,7 +436,10 @@ bool JOIN::check_for_splittable_materialized()
     }
   }
   if (candidates.elements() == 0)  // no candidates satisfying (8.1) && (8.2)
+  {
+    trace_split.add("not_applicable", "group list has no candidates");
     return false;
+  }
 
   /*
     For each table from this join find the keys that can be used for ref access
@@ -444,7 +498,11 @@ bool JOIN::check_for_splittable_materialized()
   }
 
   if (!spl_field_cnt)  // No candidate field can be accessed by ref => !(9)
+  {
+    trace_split.add("not_applicable",
+                    "no candidate field can be accessed through ref");
     return false;
+  }
 
   /*
     Create a structure of the type SplM_opt_info and fill it with
@@ -462,16 +520,22 @@ bool JOIN::check_for_splittable_materialized()
   spl_opt_info->tables_usable_for_splitting= 0;
   spl_opt_info->spl_field_cnt= spl_field_cnt;
   spl_opt_info->spl_fields= spl_field;
-  for (cand= cand_start; cand < cand_end; cand++)
+
   {
-    if (!cand->is_usable_for_ref_access)
-      continue;
-    spl_field->producing_item= cand->producing_item;
-    spl_field->underlying_field= cand->underlying_field;
-    spl_field->mat_field= cand->mat_field;
-    spl_opt_info->tables_usable_for_splitting|=
-    cand->underlying_field->table->map;
-    spl_field++;
+    Json_writer_array trace_range(thd, "split_candidates");
+    for (cand= cand_start; cand < cand_end; cand++)
+    {
+      if (!cand->is_usable_for_ref_access)
+        continue;
+      trace_range.add(cand->producing_item);
+
+      spl_field->producing_item= cand->producing_item;
+      spl_field->underlying_field= cand->underlying_field;
+      spl_field->mat_field= cand->mat_field;
+      spl_opt_info->tables_usable_for_splitting|=
+      cand->underlying_field->table->map;
+      spl_field++;
+    }
   }
 
   /* Attach this info to the table T */
@@ -600,15 +664,18 @@ add_ext_keyuse_for_splitting(Dynamic_array<KEYUSE_EXT> *ext_keyuses,
   keyuse_ext.cond_guard= added_key_field->cond_guard;
   keyuse_ext.sj_pred_no= added_key_field->sj_pred_no;
   keyuse_ext.validity_ref= 0;
-  keyuse_ext.needed_in_prefix= added_key_field->val->used_tables();
+  keyuse_ext.needed_in_prefix= added_key_field->val->used_tables() &
+			       ~(OUTER_REF_TABLE_BIT | RAND_TABLE_BIT);
   keyuse_ext.validity_var= false;
   return ext_keyuses->push(keyuse_ext);
 }
 
 
 static int
-sort_ext_keyuse(KEYUSE_EXT *a, KEYUSE_EXT *b)
+sort_ext_keyuse(const void *a_, const void *b_)
 {
+  const KEYUSE_EXT *a= static_cast<const KEYUSE_EXT *>(a_);
+  const KEYUSE_EXT *b= static_cast<const KEYUSE_EXT *>(b_);
   if (a->table->tablenr != b->table->tablenr)
     return (int) (a->table->tablenr - b->table->tablenr);
   if (a->key != b->key)
@@ -726,7 +793,7 @@ void JOIN::add_keyuses_for_splitting()
   bzero((char*) &keyuse_ext_end, sizeof(keyuse_ext_end));
   if (ext_keyuses_for_splitting->push(keyuse_ext_end))
     goto err;
-
+  // psergey-todo: trace anything here?
   spl_opt_info->unsplit_card= join_record_count;
 
   rec_len= table->s->rec_buff_length;
@@ -749,13 +816,13 @@ void JOIN::add_keyuses_for_splitting()
   if (allocate_dynamic(&keyuse, save_qep->keyuse.elements + added_keyuse_count))
     goto err;
 
-  memcpy(keyuse.buffer,
-         save_qep->keyuse.buffer,
-         (size_t) save_qep->keyuse.elements * keyuse.size_of_element);
-  keyuse.elements= save_qep->keyuse.elements;
+  idx= keyuse.elements= save_qep->keyuse.elements;
+  if (keyuse.elements)
+    memcpy(keyuse.buffer,
+           save_qep->keyuse.buffer,
+           (size_t) keyuse.elements * keyuse.size_of_element);
 
   keyuse_ext= &ext_keyuses_for_splitting->at(0);
-  idx= save_qep->keyuse.elements;
   for (i=0; i < added_keyuse_count; i++, keyuse_ext++, idx++)
   {
     set_dynamic(&keyuse, (KEYUSE *) keyuse_ext, idx);
@@ -824,13 +891,13 @@ SplM_plan_info *SplM_opt_info::find_plan(TABLE *table, uint key, uint parts)
 static
 void reset_validity_vars_for_keyuses(KEYUSE_EXT *key_keyuse_ext_start,
                                      TABLE *table, uint key,
-                                     table_map remaining_tables,
+                                     table_map excluded_tables,
                                      bool validity_val)
 {
   KEYUSE_EXT *keyuse_ext= key_keyuse_ext_start;
   do
   {
-    if (!(keyuse_ext->needed_in_prefix & remaining_tables))
+    if (!(keyuse_ext->needed_in_prefix & excluded_tables))
     {
       /*
         The enabling/disabling flags are set just in KEYUSE_EXT structures.
@@ -850,8 +917,11 @@ void reset_validity_vars_for_keyuses(KEYUSE_EXT *key_keyuse_ext_start,
     Choose the best splitting to extend the evaluated partial join
 
   @param
-    record_count      estimated cardinality of the extended partial join
+    idx               index for joined table T in current partial join P
     remaining_tables  tables not joined yet
+    spl_pd_boundary   OUT bitmap of the table from P extended by T that
+                      starts the sub-sequence of tables S from which
+                      no conditions are allowed to be pushed into T.
 
   @details
     This function is called during the search for the best execution
@@ -866,17 +936,20 @@ void reset_validity_vars_for_keyuses(KEYUSE_EXT *key_keyuse_ext_start,
     splitting the function set it as the true plan of materialization
     of the table T.
     The function caches the found plans for materialization of table T
-    together if the info what key was used for splitting. Next time when
+    together with the info what key was used for splitting. Next time when
     the optimizer prefers to use the same key the plan is taken from
     the cache of plans
 
   @retval
     Pointer to the info on the found plan that employs the pushed equalities
     if the plan has been chosen, NULL - otherwise.
+    If the function returns NULL the value of spl_param_tables is set to 0.
 */
 
-SplM_plan_info * JOIN_TAB::choose_best_splitting(double record_count,
-                                                 table_map remaining_tables)
+SplM_plan_info * JOIN_TAB::choose_best_splitting(uint idx,
+                                                 table_map remaining_tables,
+                                                 const POSITION *join_positions,
+                                                 table_map *spl_pd_boundary)
 {
   SplM_opt_info *spl_opt_info= table->spl_opt_info;
   DBUG_ASSERT(spl_opt_info != NULL);
@@ -891,7 +964,9 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(double record_count,
   SplM_plan_info *spl_plan= 0;
   uint best_key= 0;
   uint best_key_parts= 0;
-
+  table_map best_param_tables= 0L;
+  Json_writer_object trace_obj(thd, "choose_best_splitting");
+  Json_writer_array  trace_arr(thd, "considered_keys");
   /*
     Check whether there are keys that can be used to join T employing splitting
     and if so, select the best out of such keys
@@ -909,9 +984,11 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(double record_count,
       uint key= keyuse_ext->key;
       KEYUSE_EXT *key_keyuse_ext_start= keyuse_ext;
       key_part_map found_parts= 0;
+      table_map needed_in_prefix= 0;
       do
       {
-        if (keyuse_ext->needed_in_prefix & remaining_tables)
+        if (keyuse_ext->needed_in_prefix &
+            (remaining_tables | this->join->sjm_lookup_tables))
 	{
           keyuse_ext++;
           continue;
@@ -934,6 +1011,7 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(double record_count,
         KEY *key_info= table->key_info + key;
         double rec_per_key=
                  key_info->actual_rec_per_key(keyuse_ext->keypart);
+        needed_in_prefix|= keyuse_ext->needed_in_prefix;
         if (rec_per_key < best_rec_per_key)
 	{
           best_table= keyuse_ext->table;
@@ -941,6 +1019,14 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(double record_count,
 	  best_key_parts= keyuse_ext->keypart + 1;
           best_rec_per_key= rec_per_key;
           best_key_keyuse_ext_start= key_keyuse_ext_start;
+          best_param_tables= needed_in_prefix;
+           // trace table, key_name, parts, needed_tables.
+          Json_writer_object cur_index(thd);
+          cur_index.
+            add("table_name", best_table->alias.ptr()).
+            add("index", best_table->key_info[best_key].name).
+            add("rec_per_key", best_rec_per_key).
+            add("param_tables", best_param_tables);
         }
         keyuse_ext++;
       }
@@ -948,28 +1034,76 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(double record_count,
     }
     while (keyuse_ext->table == table);
   }
+  trace_arr.end();
+
   spl_opt_info->last_plan= 0;
+  double refills= DBL_MAX;
+  table_map excluded_tables= remaining_tables | this->join->sjm_lookup_tables;
   if (best_table)
   {
+    *spl_pd_boundary= this->table->map;
+    if (!best_param_tables)
+      refills= 1;
+    else
+    {
+      table_map last_found= this->table->map;
+      for (const POSITION *pos= &join_positions[idx - 1]; ; pos--)
+      {
+        if (pos->table->table->map & excluded_tables)
+          continue;
+        if (pos->partial_join_cardinality < refills)
+	{
+          *spl_pd_boundary= last_found;
+          refills= pos->partial_join_cardinality;
+        }
+        last_found= pos->table->table->map;
+        if ((last_found & best_param_tables) || pos->use_join_buffer)
+          break;
+      }
+    }
+
+    trace_obj.add("refills", refills).
+      add("spl_pd_boundary", *spl_pd_boundary);
+
     /*
       The key for splitting was chosen, look for the plan for this key
       in the cache
     */
     spl_plan= spl_opt_info->find_plan(best_table, best_key, best_key_parts);
-    if (!spl_plan &&
-	(spl_plan= (SplM_plan_info *) thd->alloc(sizeof(SplM_plan_info))) &&
-	(spl_plan->best_positions=
-	   (POSITION *) thd->alloc(sizeof(POSITION) * join->table_count)) &&
-	!spl_opt_info->plan_cache.push_back(spl_plan))
+    if (!spl_plan)
     {
       /*
         The plan for the chosen key has not been found in the cache.
         Build a new plan and save info on it in the cache
       */
+      Json_writer_array wrapper(thd, "split_plan_search");
       table_map all_table_map= (((table_map) 1) << join->table_count) - 1;
       reset_validity_vars_for_keyuses(best_key_keyuse_ext_start, best_table,
-                                      best_key, remaining_tables, true);
+                                      best_key, excluded_tables, true);
       choose_plan(join, all_table_map & ~join->const_table_map);
+
+      wrapper.end();
+      /*
+        Check that the chosen plan is really a splitting plan.
+        If not or if there is not enough memory to save the plan in the cache
+        then just return with no splitting plan.
+      */
+      POSITION *first_non_const_pos= join->best_positions + join->const_tables;
+      TABLE *table= first_non_const_pos->table->table;
+      key_map spl_keys= table->keys_usable_for_splitting;
+      if (!(first_non_const_pos->key &&
+            spl_keys.is_set(first_non_const_pos->key->key)) ||
+          !(spl_plan= (SplM_plan_info *) thd->alloc(sizeof(SplM_plan_info))) ||
+	  !(spl_plan->best_positions=
+	     (POSITION *) thd->alloc(sizeof(POSITION) * join->table_count)) ||
+	  spl_opt_info->plan_cache.push_back(spl_plan))
+      {
+        reset_validity_vars_for_keyuses(best_key_keyuse_ext_start, best_table,
+                                        best_key, excluded_tables, false);
+        trace_obj.add("split_plan_discarded", "constructed unapplicable query plan");
+        return 0;
+      }
+
       spl_plan->keyuse_ext_start= best_key_keyuse_ext_start;
       spl_plan->table= best_table;
       spl_plan->key= best_key;
@@ -990,18 +1124,34 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(double record_count,
              (char *) join->best_positions,
              sizeof(POSITION) * join->table_count);
       reset_validity_vars_for_keyuses(best_key_keyuse_ext_start, best_table,
-                                      best_key, remaining_tables, false);
+                                      best_key, excluded_tables, false);
     }
+    else
+      trace_obj.add("cached_plan_found", 1);
+
     if (spl_plan)
     {
-      if(record_count * spl_plan->cost < spl_opt_info->unsplit_cost)
+      trace_obj.
+        add("lead_table", spl_plan->table->alias.ptr()).
+        add("index",      spl_plan->table->key_info[spl_plan->key].name).
+        add("parts",      spl_plan->parts).
+        add("split_sel",  spl_plan->split_sel).
+        add("cost",       spl_plan->cost).
+        add("records",    (ha_rows) (records * spl_plan->split_sel)).
+        add("refills",    refills);
+
+      if (refills * spl_plan->cost < spl_opt_info->unsplit_cost - 0.01)
       {
         /*
           The best plan that employs splitting is cheaper than
           the plan without splitting
 	*/
         spl_opt_info->last_plan= spl_plan;
+        spl_opt_info->last_refills= refills;
+        trace_obj.add("chosen", true);
       }
+      else
+        trace_obj.add("chosen", false);
     }
   }
 
@@ -1010,11 +1160,14 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(double record_count,
   spl_plan= spl_opt_info->last_plan;
   if (spl_plan)
   {
-    startup_cost= record_count * spl_plan->cost;
+    startup_cost= spl_opt_info->last_refills * spl_plan->cost;
     records= (ha_rows) (records * spl_plan->split_sel);
   }
   else
+  {
     startup_cost= spl_opt_info->unsplit_cost;
+    *spl_pd_boundary= 0;
+  }
   return spl_plan;
 }
 
@@ -1024,16 +1177,16 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(double record_count,
     Inject equalities for splitting used by the materialization join
 
   @param
-    remaining_tables  used to filter out the equalities that cannot
-                      be pushed.
+    excluded_tables    used to filter out the equalities that are not
+                       to be pushed.
 
   @details
-    This function is called by JOIN_TAB::fix_splitting that is used
-    to fix the chosen splitting of a splittable materialized table T
-    in the final query execution plan. In this plan the table T
-    is joined just before the 'remaining_tables'. So all equalities
-    usable for splitting whose right parts do not depend on any of
-    remaining tables can be pushed into join for T.
+    This function injects equalities pushed into a derived table T for which
+    the split optimization has been chosen by the optimizer. The function
+    is called by JOIN::inject_splitting_cond_for_all_tables_with_split_opt().
+    All equalities usable for splitting T whose right parts do not depend on
+    any of the 'excluded_tables' can be pushed into the where clause of the
+    derived table T.
     The function also marks the select that specifies T as
     UNCACHEABLE_DEPENDENT_INJECTED.
 
@@ -1042,32 +1195,32 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(double record_count,
     true   on failure
 */
 
-bool JOIN::inject_best_splitting_cond(table_map remaining_tables)
+bool JOIN::inject_best_splitting_cond(table_map excluded_tables)
 {
   Item *inj_cond= 0;
-  List<Item> inj_cond_list;
+  List<Item> *inj_cond_list= &spl_opt_info->inj_cond_list;
   List_iterator<KEY_FIELD> li(spl_opt_info->added_key_fields);
   KEY_FIELD *added_key_field;
   while ((added_key_field= li++))
   {
-    if (remaining_tables & added_key_field->val->used_tables())
+    if (excluded_tables & added_key_field->val->used_tables())
       continue;
-    if (inj_cond_list.push_back(added_key_field->cond, thd->mem_root))
+    if (inj_cond_list->push_back(added_key_field->cond, thd->mem_root))
       return true;
   }
-  DBUG_ASSERT(inj_cond_list.elements);
-  switch (inj_cond_list.elements) {
+  DBUG_ASSERT(inj_cond_list->elements);
+  switch (inj_cond_list->elements) {
   case 1:
-    inj_cond= inj_cond_list.head(); break;
+    inj_cond= inj_cond_list->head(); break;
   default:
-    inj_cond= new (thd->mem_root) Item_cond_and(thd, inj_cond_list);
+    inj_cond= new (thd->mem_root) Item_cond_and(thd, *inj_cond_list);
     if (!inj_cond)
       return true;
   }
   if (inj_cond)
     inj_cond->fix_fields(thd,0);
 
-  if (inject_cond_into_where(inj_cond))
+  if (inject_cond_into_where(inj_cond->copy_andor_structure(thd)))
     return true;
 
   select_lex->uncacheable|= UNCACHEABLE_DEPENDENT_INJECTED;
@@ -1078,13 +1231,47 @@ bool JOIN::inject_best_splitting_cond(table_map remaining_tables)
 }
 
 
+/**
+  @brief
+    Test if equality is injected for split optimization
+
+  @param
+    eq_item   equality to to test
+
+  @retval
+    true    eq_item is equality injected for split optimization
+    false   otherwise
+*/
+
+bool is_eq_cond_injected_for_split_opt(Item_func_eq *eq_item)
+{
+  Item *left_item= eq_item->arguments()[0]->real_item();
+  if (left_item->type() != Item::FIELD_ITEM)
+    return false;
+  Field *field= ((Item_field *) left_item)->field;
+  if (!field->table->reginfo.join_tab)
+    return false;
+  JOIN *join= field->table->reginfo.join_tab->join;
+  if (!join->spl_opt_info)
+    return false;
+  List_iterator_fast<Item> li(join->spl_opt_info->inj_cond_list);
+  Item *item;
+  while ((item= li++))
+  {
+    if (item == eq_item)
+        return true;
+  }
+  return false;
+}
+
+
 /**
   @brief
     Fix the splitting chosen for a splittable table in the final query plan
 
   @param
     spl_plan   info on the splitting plan chosen for the splittable table T
-    remaining_tables  the table T is joined just before these tables
+    excluded_tables  tables that cannot be used in equalities pushed into T
     is_const_table    the table T is a constant table
 
   @details
@@ -1099,7 +1286,7 @@ bool JOIN::inject_best_splitting_cond(table_map remaining_tables)
 */
 
 bool JOIN_TAB::fix_splitting(SplM_plan_info *spl_plan,
-                             table_map remaining_tables,
+                             table_map excluded_tables,
                              bool is_const_table)
 {
   SplM_opt_info *spl_opt_info= table->spl_opt_info;
@@ -1107,11 +1294,10 @@ bool JOIN_TAB::fix_splitting(SplM_plan_info *spl_plan,
   JOIN *md_join= spl_opt_info->join;
   if (spl_plan && !is_const_table)
   {
+    is_split_derived= true;
     memcpy((char *) md_join->best_positions,
            (char *) spl_plan->best_positions,
            sizeof(POSITION) * md_join->table_count);
-    if (md_join->inject_best_splitting_cond(remaining_tables))
-      return true;
     /*
       This is called for a proper work of JOIN::get_best_combination()
       called for the join that materializes T
@@ -1119,7 +1305,7 @@ bool JOIN_TAB::fix_splitting(SplM_plan_info *spl_plan,
     reset_validity_vars_for_keyuses(spl_plan->keyuse_ext_start,
                                     spl_plan->table,
                                     spl_plan->key,
-                                    remaining_tables,
+                                    excluded_tables,
                                     true);
   }
   else if (md_join->save_qep)
@@ -1147,7 +1333,8 @@ bool JOIN_TAB::fix_splitting(SplM_plan_info *spl_plan,
 bool JOIN::fix_all_splittings_in_plan()
 {
   table_map prev_tables= 0;
-  table_map all_tables= (1 << table_count) - 1;
+  table_map all_tables= (table_map(1) << table_count) - 1;
+  table_map prev_sjm_lookup_tables= 0;
   for (uint tablenr= 0; tablenr < table_count; tablenr++)
   {
     POSITION *cur_pos= &best_positions[tablenr];
@@ -1155,11 +1342,76 @@ bool JOIN::fix_all_splittings_in_plan()
     if (tab->table->is_splittable())
     {
       SplM_plan_info *spl_plan= cur_pos->spl_plan;
-      if (tab->fix_splitting(spl_plan, all_tables & ~prev_tables,
+      table_map excluded_tables= (all_tables & ~prev_tables) |
+                                 prev_sjm_lookup_tables;
+                                   ;
+      if (spl_plan)
+      {
+        POSITION *pos= cur_pos;
+        table_map spl_pd_boundary= pos->spl_pd_boundary;
+        do
+	{
+          excluded_tables|= pos->table->table->map;
+        }
+        while (!((pos--)->table->table->map & spl_pd_boundary));
+      }
+      if (tab->fix_splitting(spl_plan,
+                             excluded_tables,
                              tablenr < const_tables ))
           return true;
     }
     prev_tables|= tab->table->map;
+    if (cur_pos->sj_strategy == SJ_OPT_MATERIALIZE)
+        prev_sjm_lookup_tables|= tab->table->map;
+  }
+  return false;
+}
+
+
+/**
+  @brief
+    Inject splitting conditions into WHERE of split derived
+
+  @details
+    The function calls JOIN_TAB::inject_best_splitting_cond() for each
+    materialized derived table T used in this join for which the split
+    optimization has been chosen by the optimizer. It is done in order to
+    inject equalities pushed into the where clause of the specification
+    of T that would be helpful to employ the splitting technique.
+
+  @retval
+    false  on success
+    true   on failure
+*/
+
+bool JOIN::inject_splitting_cond_for_all_tables_with_split_opt()
+{
+  table_map prev_tables= 0;
+  table_map all_tables= (table_map(1) << table_count) - 1;
+  for (uint tablenr= 0; tablenr < table_count; tablenr++)
+  {
+    POSITION *cur_pos= &best_positions[tablenr];
+    JOIN_TAB *tab= cur_pos->table;
+    prev_tables|= tab->table->map;
+    if (!(tab->table->is_splittable() && cur_pos->spl_plan))
+      continue;
+    SplM_opt_info *spl_opt_info= tab->table->spl_opt_info;
+    JOIN *join= spl_opt_info->join;
+    table_map excluded_tables= (all_tables & ~prev_tables) | sjm_lookup_tables;
+    table_map spl_pd_boundary= cur_pos->spl_pd_boundary;
+    for (POSITION *pos= cur_pos; ; pos--)
+    {
+      excluded_tables|= pos->table->table->map;
+      pos->table->no_forced_join_cache= true;
+      if (pos->table->table->map & spl_pd_boundary)
+      {
+        pos->table->split_derived_to_update|= tab->table->map;
+        break;
+      }
+    }
+
+    if (join->inject_best_splitting_cond(excluded_tables))
+      return true;
   }
   return false;
 }
