@@ -48,11 +48,12 @@ static ulonglong system_variable_hash_version= 0;
   Return variable name and length for hashing of variables.
 */
 
-static uchar *get_sys_var_length(const sys_var *var, size_t *length,
-                                 my_bool first)
+static const uchar *get_sys_var_length(const void *var_, size_t *length,
+                                       my_bool)
 {
+  auto var= static_cast<const sys_var *>(var_);
   *length= var->name.length;
-  return (uchar*) var->name.str;
+  return reinterpret_cast<const uchar *>(var->name.str);
 }
 
 sys_var_chain all_sys_vars = { NULL, NULL };
@@ -64,8 +65,9 @@ int sys_var_init()
   /* Must be already initialized. */
   DBUG_ASSERT(system_charset_info != NULL);
 
-  if (my_hash_init(PSI_INSTRUMENT_ME, &system_variable_hash, system_charset_info, 700, 0,
-                   0, (my_hash_get_key) get_sys_var_length, 0, HASH_UNIQUE))
+  if (my_hash_init(PSI_INSTRUMENT_ME, &system_variable_hash,
+                   system_charset_info, 700, 0, 0, get_sys_var_length, 0,
+                   HASH_UNIQUE))
     goto error;
 
   if (mysql_add_sys_var_chain(all_sys_vars.first))
@@ -614,8 +616,10 @@ int mysql_del_sys_var_chain(sys_var *first)
 }
 
 
-static int show_cmp(SHOW_VAR *a, SHOW_VAR *b)
+static int show_cmp(const void *a_, const void *b_)
 {
+  const SHOW_VAR *a= static_cast<const SHOW_VAR *>(a_);
+  const SHOW_VAR *b= static_cast<const SHOW_VAR *>(b_);
   return strcmp(a->name, b->name);
 }
 
@@ -1278,7 +1282,8 @@ end:
   and update it directly.
 */
 
-void set_sys_var_value_origin(void *ptr, enum sys_var::where here)
+void set_sys_var_value_origin(void *ptr, enum sys_var::where here,
+                              const char *filename)
 {
   bool found __attribute__((unused))= false;
   DBUG_ASSERT(!mysqld_server_started); // only to be used during startup
@@ -1289,6 +1294,7 @@ void set_sys_var_value_origin(void *ptr, enum sys_var::where here)
     if (var->option.value == ptr)
     {
       found= true;
+      var->origin_filename= filename;
       var->value_origin= here;
       /* don't break early, search for all matches */
     }
