@@ -116,12 +116,12 @@ int select_unit::send_data(List<Item> &values)
   if (table->no_rows_with_nulls)
     table->null_catch_flags= CHECK_ROW_FOR_NULLS_TO_REJECT;
 
-  fill_record(thd, table, table->field + addon_cnt, values, true, false);
+  fill_record(thd, table, table->field + addon_cnt, values, true, false, true);
   /* set up initial values for records to be written */
   if (addon_cnt && step == UNION_TYPE)
   {
     DBUG_ASSERT(addon_cnt == 1);
-    table->field[0]->store((longlong) curr_step, 1);
+    table->field[0]->store((ulonglong) curr_step, 1);
   }
 
   if (unlikely(thd->is_error()))
@@ -251,7 +251,27 @@ bool select_unit::send_eof()
 
 int select_union_recursive::send_data(List<Item> &values)
 {
-  int rc= select_unit::send_data(values);
+
+  int rc;
+  bool save_abort_on_warning= thd->abort_on_warning;
+  enum_check_fields save_count_cuted_fields= thd->count_cuted_fields;
+  long save_counter;
+
+  /*
+    For recursive CTE's give warnings for wrong field info
+    However, we don't do that for CREATE TABLE ... SELECT or INSERT ... SELECT
+    as the upper level code for these handles setting of abort_on_warning
+    depending on if 'IGNORE' is used.
+  */
+  if (thd->lex->sql_command != SQLCOM_CREATE_TABLE &&
+      thd->lex->sql_command != SQLCOM_INSERT_SELECT)
+    thd->abort_on_warning= thd->is_strict_mode();
+  thd->count_cuted_fields= CHECK_FIELD_WARN;
+  save_counter= thd->get_stmt_da()->set_current_row_for_warning(++row_counter);
+  rc= select_unit::send_data(values);
+  thd->get_stmt_da()->set_current_row_for_warning(save_counter);
+  thd->count_cuted_fields= save_count_cuted_fields;
+  thd->abort_on_warning= save_abort_on_warning;
 
   if (rc == 0 &&
       write_err != HA_ERR_FOUND_DUPP_KEY &&
@@ -324,6 +344,7 @@ select_unit::create_result_table(THD *thd_arg, List<Item> *column_types,
   DBUG_ASSERT(table == 0);
   tmp_table_param.init();
   tmp_table_param.field_count= column_types->elements;
+  tmp_table_param.func_count= tmp_table_param.field_count;
   tmp_table_param.bit_fields_as_long= bit_fields_as_long;
   tmp_table_param.hidden_field_count= hidden;
 
@@ -364,7 +385,8 @@ select_union_recursive::create_result_table(THD *thd_arg,
     return true;
   
   incr_table_param.init();
-  incr_table_param.field_count= column_types->elements;
+  incr_table_param.field_count= incr_table_param.func_count=
+    column_types->elements;
   incr_table_param.bit_fields_as_long= bit_fields_as_long;
   if (! (incr_table= create_tmp_table(thd_arg, &incr_table_param, *column_types,
                                       (ORDER*) 0, false, 1,
@@ -463,7 +485,7 @@ bool select_unit_ext::disable_index_if_needed(SELECT_LEX *curr_sl)
         !curr_sl->next_select()) )
   {
     is_index_enabled= false;
-    if (table->file->ha_disable_indexes(HA_KEY_SWITCH_ALL))
+    if (table->file->ha_disable_indexes(key_map(0), false))
       return false;
     table->no_keyread=1;
     return true;
@@ -525,7 +547,7 @@ int select_unit::delete_record()
   tables of JOIN - exec_tmp_table_[1 | 2].
 */
 
-void select_unit::cleanup()
+void select_unit::reset_for_next_ps_execution()
 {
   table->file->extra(HA_EXTRA_RESET_STATE);
   table->file->ha_delete_all_rows();
@@ -597,7 +619,7 @@ int select_unit_ext::send_data(List<Item> &values)
   if (table->no_rows_with_nulls)
     table->null_catch_flags= CHECK_ROW_FOR_NULLS_TO_REJECT;
 
-  fill_record(thd, table, table->field + addon_cnt, values, true, false);
+  fill_record(thd, table, table->field + addon_cnt, values, true, false, true);
   /* set up initial values for records to be written */
   if ( step == UNION_TYPE )
   {
@@ -880,11 +902,11 @@ bool select_unit_ext::send_eof()
   return (MY_TEST(error));
 }
 
-void select_union_recursive::cleanup()
+void select_union_recursive::reset_for_next_ps_execution()
 {
   if (table)
   {
-    select_unit::cleanup();
+    select_unit::reset_for_next_ps_execution();
     free_tmp_table(thd, table);
   }
 
@@ -918,6 +940,7 @@ void select_union_recursive::cleanup()
     thd->rec_tables= tab;
     tbl->derived_result= 0;
   }
+  row_counter= 0;
 }
 
 
@@ -980,7 +1003,7 @@ int select_union_direct::send_data(List<Item> &items)
   }
 
   send_records++;
-  fill_record(thd, table, table->field, items, true, false);
+  fill_record(thd, table, table->field, items, true, false, true);
   if (unlikely(thd->is_error()))
     return true; /* purecov: inspected */
 
@@ -1003,7 +1026,8 @@ bool select_union_direct::send_eof()
   // Reset for each SELECT_LEX, so accumulate here
   limit_found_rows+= thd->limit_found_rows;
 
-  if (unit->thd->lex->current_select == last_select_lex)
+  if (unit->thd->lex->current_select == last_select_lex ||
+      thd->killed == ABORT_QUERY)
   {
     thd->limit_found_rows= limit_found_rows;
 
@@ -1054,15 +1078,6 @@ st_select_lex_unit::init_prepare_fake_select_lex(THD *thd_arg,
          order= order->next)
       order->item= &order->item_ptr;
   }
-  for (ORDER *order= global_parameters()->order_list.first;
-       order;
-       order=order->next)
-  {
-    (*order->item)->walk(&Item::change_context_processor, 0,
-                         &fake_select_lex->context);
-    (*order->item)->walk(&Item::set_fake_select_as_master_processor, 0,
-                         fake_select_lex);
-  }
 }
 
 
@@ -1249,26 +1264,21 @@ bool st_select_lex_unit::join_union_item_types(THD *thd_arg,
 }
 
 
-bool init_item_int(THD* thd, Item_int* &item)
+static bool init_item_int(THD* thd, Item_int* &item)
 {
   if (!item)
   {
-    Query_arena *arena, backup_arena;
-    arena= thd->activate_stmt_arena_if_needed(&backup_arena);
-
     item= new (thd->mem_root) Item_int(thd, 0);
 
-    if (arena)
-      thd->restore_active_arena(arena, &backup_arena);
-
     if (!item)
-    return false;
+      return true;
   }
   else
   {
     item->value= 0;
   }
-  return true;
+
+  return false;
 }
 
 
@@ -1748,8 +1758,12 @@ cont:
 
       for(uint i= 0; i< hidden; i++)
       {
-        init_item_int(thd, addon_fields[i]);
-        types.push_front(addon_fields[i]);
+        if (init_item_int(thd, addon_fields[i]) ||
+            types.push_front(addon_fields[i]))
+        {
+          types.empty();
+          goto err;
+        }
         addon_fields[i]->name.str= i ? "__CNT_1" : "__CNT_2";
         addon_fields[i]->name.length= 7;
       }
@@ -2073,7 +2087,7 @@ bool st_select_lex_unit::optimize()
       /* re-enabling indexes for next subselect iteration */
       if ((union_result->force_enable_index_if_needed() || union_distinct))
       {
-        if(table->file->ha_enable_indexes(HA_KEY_SWITCH_ALL))
+        if(table->file->ha_enable_indexes(key_map(table->s->keys), false))
           DBUG_ASSERT(0);
         else
           table->no_keyread= 0;
@@ -2173,14 +2187,14 @@ bool st_select_lex_unit::exec()
         union_result->table && union_result->table->is_created())
     {
       union_result->table->file->ha_delete_all_rows();
-      union_result->table->file->ha_enable_indexes(HA_KEY_SWITCH_ALL);
+      union_result->table->file->ha_enable_indexes(key_map(table->s->keys), false);
     }
   }
 
   if (uncacheable || !item || !item->assigned() || describe)
   {
     if (!fake_select_lex && !(with_element && with_element->is_recursive))
-      union_result->cleanup();
+      union_result->reset_for_next_ps_execution();
     for (SELECT_LEX *sl= select_cursor; sl; sl= sl->next_select())
     {
       ha_rows records_at_start= 0;
@@ -2240,7 +2254,7 @@ bool st_select_lex_unit::exec()
 	{
           // This is UNION DISTINCT, so there should be a fake_select_lex
           DBUG_ASSERT(fake_select_lex != NULL);
-	  if (unlikely(table->file->ha_disable_indexes(HA_KEY_SWITCH_ALL)))
+	  if (table->file->ha_disable_indexes(key_map(0), false))
 	    DBUG_RETURN(TRUE);
 	  table->no_keyread=1;
 	}
@@ -2291,9 +2305,9 @@ bool st_select_lex_unit::exec()
           the current result.
         */
         push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
-                            ER_QUERY_EXCEEDED_ROWS_EXAMINED_LIMIT,
-                            ER_THD(thd, ER_QUERY_EXCEEDED_ROWS_EXAMINED_LIMIT),
-                            thd->accessed_rows_and_keys,
+                            ER_QUERY_RESULT_INCOMPLETE,
+                            ER_THD(thd, ER_QUERY_RESULT_INCOMPLETE),
+                            "LIMIT ROWS EXAMINED",
                             thd->lex->limit_rows_examined->val_uint());
         thd->reset_killed();
         break;
@@ -2622,7 +2636,7 @@ bool st_select_lex_unit::cleanup()
   {
     if (union_result)
     {
-      ((select_union_recursive *) union_result)->cleanup();
+      ((select_union_recursive *) union_result)->reset_for_next_ps_execution();
       delete union_result;
       union_result= 0;
     }
