@@ -1027,10 +1027,10 @@ static const String tz_SYSTEM_name("SYSTEM", 6, &my_charset_latin1);
 class Time_zone_system : public Time_zone
 {
 public:
-  Time_zone_system() {}                       /* Remove gcc warning */
-  virtual my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, uint *error_code) const;
-  virtual void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const;
-  virtual const String * get_name() const;
+  Time_zone_system() = default;                       /* Remove gcc warning */
+  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, uint *error_code) const override;
+  void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
+  const String * get_name() const override;
 };
 
 
@@ -1123,11 +1123,11 @@ Time_zone_system::get_name() const
 class Time_zone_utc : public Time_zone
 {
 public:
-  Time_zone_utc() {}                          /* Remove gcc warning */
-  virtual my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
-                                    uint *error_code) const;
-  virtual void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const;
-  virtual const String * get_name() const;
+  Time_zone_utc() = default;                          /* Remove gcc warning */
+  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
+                                    uint *error_code) const override;
+  void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
+  const String * get_name() const override;
 };
 
 
@@ -1207,9 +1207,9 @@ class Time_zone_db : public Time_zone
 {
 public:
   Time_zone_db(TIME_ZONE_INFO *tz_info_arg, const String * tz_name_arg);
-  virtual my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, uint *error_code) const;
-  virtual void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const;
-  virtual const String * get_name() const;
+  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, uint *error_code) const override;
+  void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
+  const String * get_name() const override;
 private:
   TIME_ZONE_INFO *tz_info;
   const String *tz_name;
@@ -1305,10 +1305,10 @@ class Time_zone_offset : public Time_zone
 {
 public:
   Time_zone_offset(long tz_offset_arg);
-  virtual my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
-                                    uint *error_code) const;
-  virtual void   gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const;
-  virtual const String * get_name() const;
+  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
+                                    uint *error_code) const override;
+  void   gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
+  const String * get_name() const override;
   /*
     This have to be public because we want to be able to access it from
     my_offset_tzs_get_key() function
@@ -1498,21 +1498,20 @@ public:
   they should obey C calling conventions.
 */
 
-extern "C" uchar *
-my_tz_names_get_key(Tz_names_entry *entry, size_t *length,
-                    my_bool not_used __attribute__((unused)))
+static const uchar *my_tz_names_get_key(const void *entry_, size_t *length,
+                                            my_bool)
 {
+  auto entry= static_cast<const Tz_names_entry *>(entry_);
   *length= entry->name.length();
-  return (uchar*) entry->name.ptr();
+  return reinterpret_cast<const uchar *>(entry->name.ptr());
 }
 
-extern "C" uchar *
-my_offset_tzs_get_key(Time_zone_offset *entry,
-                      size_t *length,
-                      my_bool not_used __attribute__((unused)))
+static const uchar *my_offset_tzs_get_key(const void *entry_,
+                                             size_t *length, my_bool)
 {
+  auto entry= static_cast<const Time_zone_offset *>(entry_);
   *length= sizeof(long);
-  return (uchar*) &entry->offset;
+  return reinterpret_cast<const uchar *>(&entry->offset);
 }
 
 
@@ -1621,18 +1620,18 @@ my_tz_init(THD *org_thd, const char *default_tzname, my_bool bootstrap)
   */
   if (!(thd= new THD(0)))
     DBUG_RETURN(1);
-  thd->thread_stack= (char*) &thd;
+  thd->thread_stack= (void*) &thd;              // Big stack
   thd->store_globals();
 
   /* Init all memory structures that require explicit destruction */
   if (my_hash_init(key_memory_tz_storage, &tz_names, &my_charset_latin1, 20, 0,
-                   0, (my_hash_get_key) my_tz_names_get_key, 0, 0))
+                   0, my_tz_names_get_key, 0, 0))
   {
     sql_print_error("Fatal error: OOM while initializing time zones");
     goto end;
   }
   if (my_hash_init(key_memory_tz_storage, &offset_tzs, &my_charset_latin1, 26,
-                   0, 0, (my_hash_get_key)my_offset_tzs_get_key, 0, 0))
+                   0, 0, my_offset_tzs_get_key, 0, 0))
   {
     sql_print_error("Fatal error: OOM while initializing time zones");
     my_hash_free(&tz_names);
@@ -2485,15 +2484,6 @@ MEM_ROOT tz_storage;
 char fullname[FN_REFLEN + 1];
 char *root_name_end;
 
-/*
-  known file types that exist in the zoneinfo directory that are safe to
-  silently skip
-*/
-const char *known_extensions[]= {
-  ".tab",
-  NullS
-};
-
 
 /*
   Recursively scan zoneinfo directory and print all found time zone
@@ -2590,20 +2580,19 @@ scan_tz_dir(char * name_end, uint symlink_recursion_level, uint verbose)
         else
         {
           /*
-            Some systems (like debian, opensuse etc) have description
-            files (.tab).  We skip these silently if verbose is > 0
+            Some systems (like Debian, openSUSE, etc) have non-timezone files:
+              * iso3166.tab
+              * leap-seconds.list
+              * leapseconds
+              * tzdata.zi
+              * zone.tab
+              * zone1970.tab
+            We skip these silently unless verbose > 0.
           */
           const char *current_ext= fn_ext(fullname);
-          my_bool known_ext= 0;
+          my_bool known_ext= strlen(current_ext) ||
+                             !strcmp(my_basename(fullname), "leapseconds");
 
-          for (const char **ext= known_extensions ; *ext ; ext++)
-          {
-            if (!strcmp(*ext, current_ext))
-            {
-              known_ext= 1;
-              break;
-            }
-          }
           if (verbose > 0 || !known_ext)
           {
             fflush(stdout);
@@ -2637,24 +2626,33 @@ static struct my_option my_long_options[] =
   {"help", '?', "Display this help and exit.", 0, 0, 0, GET_NO_ARG, NO_ARG,
    0, 0, 0, 0, 0, 0},
 #ifdef DBUG_OFF
-  {"debug", '#', "This is a non-debug version. Catch this and exit",
+  {"debug", '#', "This is a non-debug version. Catch this and exit.",
    0,0, 0, GET_DISABLED, OPT_ARG, 0, 0, 0, 0, 0, 0},
 #else
   {"debug", '#', "Output debug log. Often this is 'd:t:o,filename'.",
    0, 0, 0, GET_STR, OPT_ARG, 0, 0, 0, 0, 0, 0},
 #endif
-  {"leap", 'l', "Print the leap second information from the given time zone file. By convention, when --leap is used the next argument is the timezonefile",
+  {"leap", 'l', "Print the leap second information from the given time zone file. By convention, when --leap is used the next argument is the timezonefile.",
    &opt_leap, &opt_leap, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
-  {"verbose", 'v', "Write non critical warnings",
+  {"verbose", 'v', "Write non critical warnings.",
    &opt_verbose, &opt_verbose, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"version", 'V', "Output version information and exit.",
    0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
-  {"skip-write-binlog", 'S', "Do not replicate changes to time zone tables to other nodes in a Galera cluster",
+  {"skip-write-binlog", 'S', "Do not replicate changes to time zone tables to the binary log, or to other nodes in a Galera cluster (if wsrep_on=ON).",
    &opt_skip_write_binlog,&opt_skip_write_binlog, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
   { 0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}
 };
 
 
+static char **default_argv;
+
+static void free_allocated_data()
+{
+  free_defaults(default_argv);
+  my_end(0);
+}
+
+
 C_MODE_START
 static my_bool get_one_option(const struct my_option *, const char *,
                               const char *);
@@ -2666,11 +2664,21 @@ static void print_version(void)
 	 MYSQL_SERVER_VERSION,SYSTEM_TYPE,MACHINE_TYPE);
 }
 
+static const char *default_timezone_dir= "/usr/share/zoneinfo/";
+
+
 static void print_usage(void)
 {
-  fprintf(stderr, "Usage:\n");
-  fprintf(stderr, " %s [options] timezonedir\n", my_progname);
-  fprintf(stderr, " %s [options] timezonefile timezonename\n", my_progname);
+  fprintf(stdout, "Create SQL commands for loading system timezeone data for "
+          "MariaDB\n\n");
+  fprintf(stdout, "Usage:\n");
+  fprintf(stdout, " %s [options] timezonedir\n", my_progname);
+  fprintf(stdout, "or\n");
+  fprintf(stdout, " %s [options] timezonefile timezonename\n", my_progname);
+
+  fprintf(stdout, "\nA typical place for the system timezone directory is "
+          "\"%s\"\n", default_timezone_dir);
+
   print_defaults("my",load_default_groups);
   puts("");
   my_print_help(my_long_options);
@@ -2691,19 +2699,35 @@ get_one_option(const struct my_option *opt, const char *argument, const char *)
     print_version();
     puts("");
     print_usage();
+    free_allocated_data();
     exit(0);
   case 'V':
     print_version();
+    free_allocated_data();
     exit(0);
   }
   return 0;
 }
 
 
+static const char *lock_tables=
+  "LOCK TABLES time_zone WRITE,\n"
+  "  time_zone_leap_second WRITE,\n"
+  "  time_zone_name WRITE,\n"
+  "  time_zone_transition WRITE,\n"
+  "  time_zone_transition_type WRITE;\n";
+static const char *trunc_tables_const=
+  "SET @old_alter_alg=@@SESSION.alter_algorithm;\n"
+  "SET session alter_algorithm='COPY';\n"
+  "TRUNCATE TABLE time_zone;\n"
+  "TRUNCATE TABLE time_zone_name;\n"
+  "TRUNCATE TABLE time_zone_transition;\n"
+  "TRUNCATE TABLE time_zone_transition_type;\n";
+
 int
 main(int argc, char **argv)
 {
-  char **default_argv;
+  const char *trunc_tables;
   MY_INIT(argv[0]);
 
   load_defaults_or_exit("my", load_default_groups, &argc, &argv);
@@ -2715,34 +2739,42 @@ main(int argc, char **argv)
   if ((argc != 1 && argc != 2) || (opt_leap && argc != 1))
   {
     print_usage();
-    free_defaults(default_argv);
+    free_allocated_data();
     return 1;
   }
 
+  if (!(argc == 1 && !opt_leap))
+    trunc_tables= "SELECT 'skip truncate tables';\n"; // No-op - needed for ELSE clause
+  else
+    trunc_tables= trunc_tables_const;
+
   if (opt_skip_write_binlog)
-  {
     /* If skip_write_binlog is set and wsrep is compiled in we disable
        sql_log_bin and wsrep_on to avoid Galera replicating below
-       truncate table clauses. This will allow user to set different
+       TRUNCATE TABLE clauses. This will allow user to set different
        time zones to nodes in Galera cluster. */
-    printf("set @prep1=if((select count(*) from information_schema.global_variables where variable_name='wsrep_on' and variable_value='ON'), 'SET SESSION SQL_LOG_BIN=?, WSREP_ON=OFF;', 'do ?');\n"
-           "prepare set_wsrep_write_binlog from @prep1;\n"
-           "set @toggle=0; execute set_wsrep_write_binlog using @toggle;\n");
-  }
+    printf("set @prep1=if((select count(*) from information_schema.global_variables where variable_name='wsrep_on' and variable_value='ON'), 'SET SESSION WSREP_ON=OFF', 'do 0');\n"
+           "SET SESSION SQL_LOG_BIN=0;\n"
+           "execute immediate @prep1;\n"
+           "%s%s", trunc_tables, lock_tables);
   else
-  {
-      // Alter time zone tables to InnoDB if wsrep_on is enabled
-      // to allow changes to them to replicate with Galera
-      printf("\\d |\n"
-        "IF (select count(*) from information_schema.global_variables where\n"
-        "variable_name='wsrep_on' and variable_value='ON') = 1 THEN\n"
-        "ALTER TABLE time_zone ENGINE=InnoDB;\n"
-        "ALTER TABLE time_zone_name ENGINE=InnoDB;\n"
-        "ALTER TABLE time_zone_transition ENGINE=InnoDB;\n"
-        "ALTER TABLE time_zone_transition_type ENGINE=InnoDB;\n"
-        "END IF|\n"
-        "\\d ;\n");
-  }
+    // Alter time zone tables to InnoDB if wsrep_on is enabled
+    // to allow changes to them to replicate with Galera
+    printf("\\d |\n"
+      "IF (select count(*) from information_schema.global_variables where\n"
+      "variable_name='wsrep_on' and variable_value='ON') = 1 THEN\n"
+      "ALTER TABLE time_zone ENGINE=InnoDB;\n"
+      "ALTER TABLE time_zone_name ENGINE=InnoDB;\n"
+      "ALTER TABLE time_zone_transition ENGINE=InnoDB;\n"
+      "ALTER TABLE time_zone_transition_type ENGINE=InnoDB;\n"
+      "%s"
+      "START TRANSACTION;\n"
+      "ELSE\n%s"
+      "END IF|\n"
+      "\\d ;\n",
+      trunc_tables, trunc_tables);
+    // Ideally we'd like to put lock_tables in the ELSE branch however
+    // "ERROR 1314 (0A000) at line 2: LOCK is not allowed in stored procedures"
 
   if (argc == 1 && !opt_leap)
   {
@@ -2750,12 +2782,6 @@ main(int argc, char **argv)
 
     root_name_end= strmake_buf(fullname, argv[0]);
 
-    printf("TRUNCATE TABLE time_zone;\n");
-    printf("TRUNCATE TABLE time_zone_name;\n");
-    printf("TRUNCATE TABLE time_zone_transition;\n");
-    printf("TRUNCATE TABLE time_zone_transition_type;\n");
-    printf("START TRANSACTION;\n");
-
     if (scan_tz_dir(root_name_end, 0, opt_verbose))
     {
       printf("ROLLBACK;\n");
@@ -2766,7 +2792,8 @@ main(int argc, char **argv)
       return 1;
     }
 
-    printf("COMMIT;\n");
+    printf("UNLOCK TABLES;\n"
+           "COMMIT;\n");
     printf("ALTER TABLE time_zone_transition "
            "ORDER BY Time_zone_id, Transition_time;\n");
     printf("ALTER TABLE time_zone_transition_type "
@@ -2790,25 +2817,27 @@ main(int argc, char **argv)
       print_tz_leaps_as_sql(&tz_info);
     else
       print_tz_as_sql(argv[1], &tz_info);
-
+    printf("UNLOCK TABLES;\n"
+           "COMMIT;\n");
     free_root(&tz_storage, MYF(0));
   }
 
   if(!opt_skip_write_binlog)
-  {
-      // Fall back to Aria
-      printf("\\d |\n"
-        "IF (select count(*) from information_schema.global_variables where\n"
-        "variable_name='wsrep_on' and variable_value='ON') = 1 THEN\n"
-        "ALTER TABLE time_zone ENGINE=Aria;\n"
-        "ALTER TABLE time_zone_name ENGINE=Aria;\n"
-        "ALTER TABLE time_zone_transition ENGINE=Aria;\n"
-        "ALTER TABLE time_zone_transition_type ENGINE=Aria;\n"
-        "END IF|\n"
-        "\\d ;\n");
-  }
+    // Fall back to Aria
+    printf("\\d |\n"
+      "IF (select count(*) from information_schema.global_variables where\n"
+      "variable_name='wsrep_on' and variable_value='ON') = 1 THEN\n"
+      "ALTER TABLE time_zone ENGINE=Aria;\n"
+      "ALTER TABLE time_zone_name ENGINE=Aria;\n"
+      "ALTER TABLE time_zone_transition ENGINE=Aria, ORDER BY Time_zone_id, Transition_time;\n"
+      "ALTER TABLE time_zone_transition_type ENGINE=Aria, ORDER BY Time_zone_id, Transition_type_id;\n"
+      "END IF|\n"
+      "\\d ;\n");
 
-  free_defaults(default_argv);
+  if (argc == 1 && !opt_leap)
+    printf("SET session alter_algorithm=@old_alter_alg;\n");
+  
+  free_allocated_data();
   my_end(0);
   return 0;
 }
