@@ -69,6 +69,7 @@
 #include "liboamcpp.h"
 #include "crashtrace.h"
 #include "utils_utf8.h"
+#include "service.h"
 
 #include <mutex>
 #include <thread>
@@ -76,7 +77,80 @@
 
 #include "dbrm.h"
 
-#include "collation.h"
+#include "mariadb_my_sys.h"
+
+
+class Opt
+{
+public:
+    int m_debug;
+    bool m_e;
+    bool m_fg;
+    Opt(int argc, char *argv[])
+     :m_debug(0),
+      m_e(false),
+      m_fg(false)
+    {
+        int c;
+        while ((c = getopt(argc, argv, "edf")) != EOF)
+        {
+            switch (c)
+            {
+                case 'd':
+                    m_debug++;
+                    break;
+
+                case 'e':
+                    m_e= true;
+                    break;
+
+                case 'f':
+                    m_fg= true;
+                    break;
+
+                case '?':
+                default:
+                    break;
+            }
+        }
+    }
+};
+
+
+class ServiceExeMgr: public Service, public Opt
+{
+protected:
+
+    void log(logging::LOG_TYPE type, const std::string &str)
+    {
+        logging::LoggingID logid(16);
+        logging::Message::Args args;
+        logging::Message message(8);
+        args.add(strerror(errno));
+        message.format(args);
+        logging::Logger logger(logid.fSubsysID);
+        logger.logMessage(type, message, logid);
+    }
+
+public:
+    ServiceExeMgr(const Opt &opt)
+     :Service("ExeMgr"), Opt(opt)
+    { }
+    void LogErrno() override
+    {
+        log(logging::LOG_TYPE_CRITICAL, std::string(strerror(errno)));
+    }
+    void ParentLogChildMessage(const std::string &str) override
+    {
+        log(logging::LOG_TYPE_INFO, str);
+    }
+    int Child() override;
+    int Run()
+    {
+        return m_fg ? Child() : RunForking();
+    }
+};
+
 
 namespace
 {
@@ -1280,27 +1354,9 @@ void added_a_pm(int)
 
     if (ec)
     {
-        //set BUSY_INIT state while processing the add pm configuration change
-        oam::Oam oam;
-
-        try
-        {
-            oam.processInitComplete("ExeMgr", oam::BUSY_INIT);
-        }
-        catch (...)
-        {}
-
         oam::OamCache* oamCache = oam::OamCache::makeOamCache();
         oamCache->forceReload();
         ec->Setup();
-
-        //set ACTIVE state
-        try
-        {
-            oam.processInitComplete("ExeMgr");
-        }
-        catch (...)
-        {}
     }
 }
 
@@ -1389,110 +1445,105 @@ int setupResources()
 
 void cleanTempDir()
 {
-    const auto config = config::Config::makeConfig();
-    std::string allowDJS = config->getConfig("HashJoin", "AllowDiskBasedJoin");
-    std::string tmpPrefix = config->getConfig("HashJoin", "TempFilePath");
+  using TempDirPurpose = config::Config::TempDirPurpose;
+  struct Dirs
+  {
+    std::string section;
+    std::string allowed;
+    TempDirPurpose purpose;
+  };
+  std::vector<Dirs> dirs{
+      {
+          "HashJoin",
+          "AllowDiskBasedJoin",
+          TempDirPurpose::Joins
+      },
+      {
+          "RowAggregation",
+          "AllowDiskBasedAggregation",
+          TempDirPurpose::Aggregates
+      }
+  };
+  const auto config = config::Config::makeConfig();
 
-    if (allowDJS == "N" || allowDJS == "n")
-        return;
+  for (const auto& dir : dirs)
+  {
+    std::string allowStr = config->getConfig(dir.section, dir.allowed);
+    bool allow = (allowStr == "Y" || allowStr == "y");
 
-    if (tmpPrefix.empty())
-        tmpPrefix = "/tmp/cs-diskjoin";
+    std::string tmpPrefix = config->getTempFileDir(dir.purpose);
+
+    if (allow && tmpPrefix.empty())
+    {
+      std::cerr << "Empty tmp directory name for " << dir.section << std::endl;
+      logging::LoggingID logid(16, 0, 0);
+      logging::Message::Args args;
+      logging::Message message(8);
+      args.add("Empty tmp directory name for:");
+      args.add(dir.section);
+      message.format(args);
+      logging::Logger logger(logid.fSubsysID);
+      logger.logMessage(logging::LOG_TYPE_CRITICAL, message, logid);
+    }
 
     tmpPrefix += "/";
 
-    assert(tmpPrefix != "/");
+    idbassert(tmpPrefix != "/");
 
     /* This is quite scary as ExeMgr usually runs as root */
     try
     {
+      if (allow)
+      {
         boost::filesystem::remove_all(tmpPrefix);
-        boost::filesystem::create_directories(tmpPrefix);
+      }
+      boost::filesystem::create_directories(tmpPrefix);
     }
-    catch (const std::exception& ex)
+    catch (const std::exception &ex)
     {
-        std::cerr << ex.what() << std::endl;
-        logging::LoggingID logid(16, 0, 0);
-        logging::Message::Args args;
-        logging::Message message(8);
-        args.add("Execption whilst cleaning tmpdir: ");
-        args.add(ex.what());
-        message.format( args );
-        logging::Logger logger(logid.fSubsysID);
-        logger.logMessage(logging::LOG_TYPE_WARNING, message, logid);
+      std::cerr << ex.what() << std::endl;
+      logging::LoggingID logid(16, 0, 0);
+      logging::Message::Args args;
+      logging::Message message(8);
+      args.add("Exception whilst cleaning tmpdir: ");
+      args.add(ex.what());
+      message.format(args);
+      logging::Logger logger(logid.fSubsysID);
+      logger.logMessage(logging::LOG_TYPE_WARNING, message, logid);
     }
     catch (...)
     {
-        std::cerr << "Caught unknown exception during tmpdir cleanup" << std::endl;
-        logging::LoggingID logid(16, 0, 0);
-        logging::Message::Args args;
-        logging::Message message(8);
-        args.add("Unknown execption whilst cleaning tmpdir");
-        message.format( args );
-        logging::Logger logger(logid.fSubsysID);
-        logger.logMessage(logging::LOG_TYPE_WARNING, message, logid);
+      std::cerr << "Caught unknown exception during tmpdir cleanup"
+                << std::endl;
+      logging::LoggingID logid(16, 0, 0);
+      logging::Message::Args args;
+      logging::Message message(8);
+      args.add("Unknown exception whilst cleaning tmpdir");
+      message.format(args);
+      logging::Logger logger(logid.fSubsysID);
+      logger.logMessage(logging::LOG_TYPE_WARNING, message, logid);
     }
+  }
 }
 
 
-int main(int argc, char* argv[])
+int ServiceExeMgr::Child()
 {
-    // Set locale language
-    setlocale(LC_ALL, "");
-    setlocale(LC_NUMERIC, "C");
-    // Initialize the charset library
-    my_init();
 
-    // This is unset due to the way we start it
-    program_invocation_short_name = const_cast<char*>("ExeMgr");
-
-    gDebug = 0;
-    bool eFlg = false;
-    int c;
-
-    opterr = 0;
-
-    while ((c = getopt(argc, argv, "ed")) != EOF)
-        switch (c)
-        {
-            case 'd':
-                gDebug++;
-                break;
-
-            case 'e':
-                eFlg = true;
-                break;
-
-            case '?':
-            default:
-                break;
-        }
-
-    //set BUSY_INIT state
-    {
-        oam::Oam oam;
-
-        try
-        {
-            oam.processInitComplete("ExeMgr", oam::BUSY_INIT);
-        }
-        catch (...)
-        {
-        }
-    }
+    gDebug= m_debug;
 
 #ifdef _MSC_VER
     //FIXME:
 #else
 
     // Make sure CSC thinks it's on a UM or else bucket reuse stuff below will stall
-    if (!eFlg)
+    if (!m_e)
         setenv("CALPONT_CSC_IDENT", "um", 1);
 
 #endif
     setupSignalHandlers();
     int err = 0;
-    if (!gDebug)
+    if (!m_debug)
         err = setupResources();
     std::string errMsg;
 
@@ -1530,14 +1581,7 @@ int main(int argc, char* argv[])
         ml.logCriticalMessage( message );
         std::cerr << errMsg << std::endl;
 
-        try
-        {
-            oam.processInitFailure();
-        }
-        catch (...)
-        {
-        }
-
+        NotifyServiceInitializationFailed();
         return 2;
     }
 
@@ -1633,23 +1677,12 @@ int main(int argc, char* argv[])
         }
     }
 
+    NotifyServiceStarted();
+
     std::cout << "Starting ExeMgr: st = " << serverThreads <<
          ", qs = " << rm->getEmExecQueueSize() << ", mx = " << maxPct << ", cf = " <<
          rm->getConfig()->configFile() << std::endl;
 
-    //set ACTIVE state
-    {
-        oam::Oam oam;
-
-        try
-        {
-            oam.processInitComplete("ExeMgr");
-        }
-        catch (...)
-        {
-        }
-    }
-
     {
         BRM::DBRM *dbrm = new BRM::DBRM();
         dbrm->setSystemQueryReady(true);
@@ -1676,4 +1709,24 @@ int main(int argc, char* argv[])
 
     return 0;
 }
+
+
+int main(int argc, char* argv[])
+{
+    opterr = 0;
+    Opt opt(argc, argv);
+
+    // Set locale language
+    setlocale(LC_ALL, "");
+    setlocale(LC_NUMERIC, "C");
+
+    // This is unset due to the way we start it
+    program_invocation_short_name = const_cast<char*>("ExeMgr");
+
+    // Initialize the charset library
+    my_init();
+
+    return ServiceExeMgr(opt).Run();
+}
+
 // vim:ts=4 sw=4:
