@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1995, 2016, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2017, 2021, MariaDB Corporation.
+Copyright (c) 2017, 2022, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -1054,7 +1054,7 @@ static
 buf_block_t*
 fsp_page_create(fil_space_t *space, page_no_t offset, mtr_t *mtr)
 {
-  buf_block_t *block, *free_block;
+  buf_block_t *block;
 
   if (UNIV_UNLIKELY(space->is_being_truncated))
   {
@@ -1063,26 +1063,48 @@ fsp_page_create(fil_space_t *space, page_no_t offset, mtr_t *mtr)
     mysql_mutex_lock(&buf_pool.mutex);
     block= reinterpret_cast<buf_block_t*>
       (buf_pool.page_hash_get_low(page_id, fold));
-    if (block && block->page.oldest_modification() <= 1)
-      block= nullptr;
+    if (!block)
+    {
+      mysql_mutex_unlock(&buf_pool.mutex);
+      goto create;
+    }
+
+    buf_block_buf_fix_inc(block, __FILE__, __LINE__);
     mysql_mutex_unlock(&buf_pool.mutex);
 
-    if (block)
+    ut_ad(block->page.state() == BUF_BLOCK_FILE_PAGE);
+    ut_ad(!block->page.ibuf_exist);
+#ifdef BTR_CUR_HASH_ADAPT
+    ut_ad(!block->index);
+#endif
+
+    if (mtr->have_x_latch(*block))
     {
+      buf_block_buf_fix_dec(block);
       ut_ad(block->page.buf_fix_count() >= 1);
       ut_ad(rw_lock_get_x_lock_count(&block->lock) == 1);
-      ut_ad(mtr->have_x_latch(*block));
-      free_block= block;
-      goto got_free_block;
+    }
+    else
+    {
+      rw_lock_x_lock(&block->lock);
+      if (UNIV_UNLIKELY(block->page.id() != page_id))
+      {
+        buf_block_buf_fix_dec(block);
+        rw_lock_x_unlock(&block->lock);
+        goto create;
+      }
+      mtr->memo_push(block, MTR_MEMO_PAGE_X_FIX);
     }
   }
-
-  free_block= buf_LRU_get_free_block(false);
-got_free_block:
-  block= buf_page_create(space, static_cast<uint32_t>(offset),
-                         space->zip_size(), mtr, free_block);
-  if (UNIV_UNLIKELY(block != free_block))
-    buf_pool.free_block(free_block);
+  else
+  {
+  create:
+    buf_block_t *free_block= buf_LRU_get_free_block(false);
+    block= buf_page_create(space, static_cast<uint32_t>(offset),
+                           space->zip_size(), mtr, free_block);
+    if (UNIV_UNLIKELY(block != free_block))
+      buf_pool.free_block(free_block);
+  }
 
   fsp_init_file_page(space, block, mtr);
   return block;
@@ -1676,6 +1698,7 @@ fseg_create(fil_space_t *space, ulint byte_offset, mtr_t *mtr,
 	fseg_inode_t*	inode;
 	ib_id_t		seg_id;
 	uint32_t	n_reserved;
+	bool		reserved_extent = false;
 
 	DBUG_ENTER("fseg_create");
 
@@ -1700,18 +1723,35 @@ fseg_create(fil_space_t *space, ulint byte_offset, mtr_t *mtr,
 		}
 	}
 
-	if (!has_done_reservation
-	    && !fsp_reserve_free_extents(&n_reserved, space, 2,
-					 FSP_NORMAL, mtr)) {
-		DBUG_RETURN(NULL);
-	}
-
 	buf_block_t* header = fsp_get_header(space, mtr);
 	buf_block_t* iblock;
 
+inode_alloc:
 	inode = fsp_alloc_seg_inode(space, header, &iblock, mtr);
 
 	if (inode == NULL) {
+reserve_extent:
+		if (!has_done_reservation && !reserved_extent) {
+
+			if (!fsp_reserve_free_extents(
+					&n_reserved, space, 2,
+					FSP_NORMAL, mtr)) {
+				DBUG_RETURN(NULL);
+			}
+
+			/* Extents reserved successfully. So
+			try allocating the page or inode */
+			reserved_extent = true;
+			if (inode) {
+				goto page_alloc;
+			}
+
+			goto inode_alloc;
+		}
+
+		if (inode) {
+			fsp_free_seg_inode(space, inode, iblock, mtr);
+		}
 		goto funct_exit;
 	}
 
@@ -1737,6 +1777,7 @@ fseg_create(fil_space_t *space, ulint byte_offset, mtr_t *mtr,
 		    FSEG_FRAG_SLOT_SIZE * FSEG_FRAG_ARR_N_SLOTS, 0xff);
 
 	if (!block) {
+page_alloc:
 		block = fseg_alloc_free_page_low(space,
 						 inode, iblock, 0, FSP_UP,
 #ifdef UNIV_DEBUG
@@ -1744,13 +1785,9 @@ fseg_create(fil_space_t *space, ulint byte_offset, mtr_t *mtr,
 #endif /* UNIV_DEBUG */
 						 mtr, mtr);
 
-		/* The allocation cannot fail if we have already reserved a
-		space for the page. */
-		ut_ad(!has_done_reservation || block != NULL);
-
-		if (block == NULL) {
-			fsp_free_seg_inode(space, inode, iblock, mtr);
-			goto funct_exit;
+		if (!block) {
+			ut_ad(!has_done_reservation);
+			goto reserve_extent;
 		}
 
 		ut_d(const auto x = rw_lock_get_x_lock_count(&block->lock));
@@ -1772,7 +1809,7 @@ fseg_create(fil_space_t *space, ulint byte_offset, mtr_t *mtr,
 				       + block->frame, space->id);
 
 funct_exit:
-	if (!has_done_reservation) {
+	if (!has_done_reservation && reserved_extent) {
 		space->release_free_extents(n_reserved);
 	}
 
@@ -2668,14 +2705,23 @@ fseg_free_extent(
 			      not_full_n_used - descr_n_used);
 	}
 
-	fsp_free_extent(space, page, mtr);
+	std::vector<uint8_t> going_to_free;
+	static_assert(FSP_EXTENT_SIZE_MIN == 256, "compatibility");
+	static_assert(FSP_EXTENT_SIZE_MAX == 64, "compatibility");
 
 	for (uint32_t i = 0; i < FSP_EXTENT_SIZE; i++) {
 		if (!xdes_is_free(descr, i)) {
-			buf_page_free(space, first_page_in_extent + i, mtr,
-				      __FILE__, __LINE__);
+			going_to_free.emplace_back(uint8_t(i));
 		}
 	}
+
+	fsp_free_extent(space, page, mtr);
+
+	for (uint32_t i : going_to_free) {
+		mtr->free(*space, first_page_in_extent + i);
+		buf_page_free(space, first_page_in_extent + i, mtr,
+			      __FILE__, __LINE__);
+	}
 }
 
 /**********************************************************************//**
