@@ -590,9 +590,10 @@ private:
 
 public:
   /** Try to close a file to adhere to the innodb_open_files limit.
+  @param ignore_space Ignore the tablespace which is acquired by caller
   @param print_info   whether to diagnose why a file cannot be closed
   @return whether a file was closed */
-  static bool try_to_close(bool print_info);
+  static bool try_to_close(fil_space_t *ignore_space, bool print_info);
 
   /** Close all tablespace files at shutdown */
   static void close_all();
@@ -1059,17 +1060,12 @@ struct fil_node_t final
 		return(handle != OS_FILE_CLOSED);
 	}
 
-	/** Read the first page of a data file.
-	@return	whether the page was found valid */
-	bool read_page0();
+  /** Read the first page of a data file.
+  @return whether the page was found valid */
+  bool read_page0() noexcept;
 
-	/** Determine some file metadata when creating or reading the file.
-	@param	file	the file that is being created, or OS_FILE_CLOSED */
-	void find_metadata(os_file_t file = OS_FILE_CLOSED
-#ifndef _WIN32
-			   , struct stat* statbuf = NULL
-#endif
-			   );
+  /** Determine some file metadata when creating or reading the file. */
+  void find_metadata() noexcept;
 
   /** Close the file handle. */
   void close();
