@@ -2,7 +2,7 @@
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All rights reserved.
 Copyright (c) 2009, Google Inc.
-Copyright (c) 2017, 2021, MariaDB Corporation.
+Copyright (c) 2017, 2022, MariaDB Corporation.
 
 Portions of this file contain modifications contributed and copyrighted by
 Google, Inc. Those modifications are gratefully acknowledged and are described
@@ -346,26 +346,6 @@ or the MySQL version that created the redo log file. */
 					header */
 #define LOG_FILE_HDR_SIZE	(4 * OS_FILE_LOG_BLOCK_SIZE)
 
-/** Memory mapped file */
-class mapped_file_t
-{
-public:
-  mapped_file_t()= default;
-  mapped_file_t(const mapped_file_t &)= delete;
-  mapped_file_t &operator=(const mapped_file_t &)= delete;
-  mapped_file_t(mapped_file_t &&)= delete;
-  mapped_file_t &operator=(mapped_file_t &&)= delete;
-  ~mapped_file_t() noexcept;
-
-  dberr_t map(const char *path, bool read_only= false,
-              bool nvme= false) noexcept;
-  dberr_t unmap() noexcept;
-  byte *data() noexcept { return m_area.data(); }
-
-private:
-  span<byte> m_area;
-};
-
 /** Abstraction for reading, writing and flushing file cache to disk */
 class file_io
 {
@@ -398,14 +378,14 @@ public:
   file_os_io &operator=(file_os_io &&rhs);
   ~file_os_io() noexcept;
 
-  dberr_t open(const char *path, bool read_only) noexcept final;
+  dberr_t open(const char *path, bool read_only) noexcept override final;
   bool is_opened() const noexcept { return m_fd != OS_FILE_CLOSED; }
-  dberr_t rename(const char *old_path, const char *new_path) noexcept final;
-  dberr_t close() noexcept final;
-  dberr_t read(os_offset_t offset, span<byte> buf) noexcept final;
+  dberr_t rename(const char *old_path, const char *new_path) noexcept override final;
+  dberr_t close() noexcept override final;
+  dberr_t read(os_offset_t offset, span<byte> buf) noexcept override final;
   dberr_t write(const char *path, os_offset_t offset,
-                span<const byte> buf) noexcept final;
-  dberr_t flush() noexcept final;
+                span<const byte> buf) noexcept override final;
+  dberr_t flush() noexcept override final;
 
 private:
   pfs_os_file_t m_fd{OS_FILE_CLOSED};
@@ -480,6 +460,11 @@ public:
   size_t buf_free;
   /** recommended maximum size of buf, after which the buffer is flushed */
   size_t max_buf_free;
+
+  /** Log sequence number when a log file overwrite (broken crash recovery)
+  was noticed. Protected by mutex. */
+  lsn_t overwrite_warned;
+
   /** mutex to serialize access to the flush list when we are putting
   dirty blocks in the list. The idea behind this mutex is to be able
   to release log_sys.mutex during mtr_commit and still ensure that
@@ -518,12 +503,16 @@ public:
     void write_header_durable(lsn_t lsn);
     /** opens log file which must be closed prior this call */
     dberr_t rename(std::string path) { return fd.rename(path); }
+
+    MY_ATTRIBUTE((warn_unused_result))
     /** reads buffer from log file
     @param[in]	offset		offset in log file
     @param[in]	buf		buffer where to read */
-    void read(os_offset_t offset, span<byte> buf);
+    dberr_t read(os_offset_t offset, span<byte> buf)
+    { return fd.read(offset, buf); }
     /** Tells whether writes require calling flush() */
-    bool writes_are_durable() const noexcept;
+    bool writes_are_durable() const noexcept
+    { return fd.writes_are_durable(); }
     /** writes buffer to log file
     @param[in]	offset		offset in log file
     @param[in]	buf		buffer from which to write */
@@ -615,11 +604,10 @@ public:
 					/*!< next checkpoint number */
   /** latest completed checkpoint (protected by log_sys.mutex) */
   Atomic_relaxed<lsn_t> last_checkpoint_lsn;
-	lsn_t		next_checkpoint_lsn;
-					/*!< next checkpoint lsn */
-	ulint		n_pending_checkpoint_writes;
-					/*!< number of currently pending
-					checkpoint writes */
+  /** next checkpoint LSN (protected by log_sys.mutex) */
+  lsn_t next_checkpoint_lsn;
+  /** whether a checkpoint is pending */
+  Atomic_relaxed<bool> checkpoint_pending;
 
   /** buffer for checkpoint header */
   byte *checkpoint_buf;
@@ -697,8 +685,9 @@ public:
     return flushes.load(std::memory_order_relaxed);
   }
 
-  /** Initialise the redo log subsystem. */
-  void create();
+  /** Initialise the redo log subsystem.
+  @return whether the initialisation succeeded */
+  bool create();
 
   /** Shut down the redo log subsystem. */
   void close();
@@ -750,6 +739,6 @@ inline void log_t::file::set_lsn_offset(lsn_t a_lsn)
   lsn_offset= a_lsn;
 }
 
-#include "log0log.ic"
+#include "log0log.inl"
 
 #endif
