@@ -221,6 +221,7 @@ trx_purge_add_undo_to_history(const trx_t* trx, trx_undo_t*& undo, mtr_t* mtr)
 	trx_ulogf_t*	undo_header	= undo_page->frame + undo->hdr_offset;
 
 	ut_ad(mach_read_from_2(undo_header + TRX_UNDO_NEEDS_PURGE) <= 1);
+	ut_ad(rseg->needs_purge > trx->id);
 
 	if (UNIV_UNLIKELY(mach_read_from_4(TRX_RSEG + TRX_RSEG_FORMAT
 					   + rseg_header->frame))) {
@@ -309,7 +310,6 @@ trx_purge_add_undo_to_history(const trx_t* trx, trx_undo_t*& undo, mtr_t* mtr)
 		rseg->last_page_no = undo->hdr_page_no;
 		rseg->set_last_commit(undo->hdr_offset,
 				      trx->rw_trx_hash_element->no);
-		rseg->needs_purge = true;
 	}
 
 	trx_sys.rseg_history_len++;
@@ -339,91 +339,67 @@ static void trx_purge_remove_log_hdr(buf_block_t *rseg, buf_block_t* log,
 }
 
 /** Free an undo log segment, and remove the header from the history list.
+@param[in,out]	mtr		mini-transaction
 @param[in,out]	rseg		rollback segment
 @param[in]	hdr_addr	file address of log_hdr */
 static
-void
-trx_purge_free_segment(trx_rseg_t* rseg, fil_addr_t hdr_addr)
+void trx_purge_free_segment(mtr_t &mtr, trx_rseg_t* rseg, fil_addr_t hdr_addr)
 {
-	mtr_t		mtr;
+  mtr.commit();
+  mtr.start();
+  ut_ad(mutex_own(&rseg->mutex));
 
-	mtr.start();
-	mutex_enter(&rseg->mutex);
+  buf_block_t *rseg_hdr= trx_rsegf_get(rseg->space, rseg->page_no, &mtr);
+  buf_block_t *block=
+    trx_undo_page_get(page_id_t(rseg->space->id, hdr_addr.page), &mtr);
+  const uint32_t seg_size=
+    flst_get_len(TRX_UNDO_SEG_HDR + TRX_UNDO_PAGE_LIST + block->frame);
+  ut_ad(rseg->curr_size >= seg_size);
+  rseg->curr_size-= seg_size;
 
-	buf_block_t* rseg_hdr = trx_rsegf_get(rseg->space, rseg->page_no, &mtr);
-	buf_block_t* block = trx_undo_page_get(
-		page_id_t(rseg->space->id, hdr_addr.page), &mtr);
+  trx_purge_remove_log_hdr(rseg_hdr, block, hdr_addr.boffset, &mtr);
+  byte *hist= TRX_RSEG + TRX_RSEG_HISTORY_SIZE + rseg_hdr->frame;
+  ut_ad(mach_read_from_4(hist) >= seg_size);
+  mtr.write<4>(*rseg_hdr, hist, mach_read_from_4(hist) - seg_size);
 
-	/* Mark the last undo log totally purged, so that if the
-	system crashes, the tail of the undo log will not get accessed
-	again. The list of pages in the undo log tail gets
-	inconsistent during the freeing of the segment, and therefore
-	purge should not try to access them again. */
-	mtr.write<2,mtr_t::MAYBE_NOP>(*block, block->frame + hdr_addr.boffset
-				      + TRX_UNDO_NEEDS_PURGE, 0U);
+  while (!fseg_free_step_not_header(TRX_UNDO_SEG_HDR + TRX_UNDO_FSEG_HEADER +
+                                    block->frame, &mtr))
+  {
+    buf_block_buf_fix_inc(rseg_hdr, __FILE__, __LINE__);
+    buf_block_buf_fix_inc(block, __FILE__, __LINE__);
+    ut_d(const page_id_t rseg_hdr_id{rseg_hdr->page.id()});
+    ut_d(const page_id_t id{block->page.id()});
+    mtr.commit();
+    /* NOTE: If the server is killed after the log that was produced
+    up to this point was written, and before the log from the mtr.commit()
+    in our caller is written, then the pages belonging to the
+    undo log will become unaccessible garbage.
 
-	while (!fseg_free_step_not_header(
-		       TRX_UNDO_SEG_HDR + TRX_UNDO_FSEG_HEADER
-		       + block->frame, &mtr)) {
-		mutex_exit(&rseg->mutex);
+    This does not matter when using multiple innodb_undo_tablespaces;
+    innodb_undo_log_truncate=ON will be able to reclaim the space. */
+    mtr.start();
+    rw_lock_x_lock(&rseg_hdr->lock);
+    rw_lock_x_lock(&block->lock);
+    ut_ad(rseg_hdr->page.id() == rseg_hdr_id);
+    ut_ad(block->page.id() == id);
+    mtr_memo_push(&mtr, rseg_hdr, MTR_MEMO_PAGE_X_FIX);
+    mtr_memo_push(&mtr, block, MTR_MEMO_PAGE_X_FIX);
+  }
 
-		mtr.commit();
-		mtr.start();
-
-		mutex_enter(&rseg->mutex);
-
-		rseg_hdr = trx_rsegf_get(rseg->space, rseg->page_no, &mtr);
-
-		block = trx_undo_page_get(
-			page_id_t(rseg->space->id, hdr_addr.page), &mtr);
-	}
-
-	/* The page list may now be inconsistent, but the length field
-	stored in the list base node tells us how big it was before we
-	started the freeing. */
-
-	const uint32_t seg_size = flst_get_len(
-		TRX_UNDO_SEG_HDR + TRX_UNDO_PAGE_LIST + block->frame);
-
-	/* We may free the undo log segment header page; it must be freed
-	within the same mtr as the undo log header is removed from the
-	history list: otherwise, in case of a database crash, the segment
-	could become inaccessible garbage in the file space. */
-
-	trx_purge_remove_log_hdr(rseg_hdr, block, hdr_addr.boffset, &mtr);
-
-	do {
-
-		/* Here we assume that a file segment with just the header
-		page can be freed in a few steps, so that the buffer pool
-		is not flooded with bufferfixed pages: see the note in
-		fsp0fsp.cc. */
-
-	} while (!fseg_free_step(TRX_UNDO_SEG_HDR + TRX_UNDO_FSEG_HEADER
-				 + block->frame, &mtr));
-
-	byte* hist = TRX_RSEG + TRX_RSEG_HISTORY_SIZE + rseg_hdr->frame;
-	ut_ad(mach_read_from_4(hist) >= seg_size);
-
-	mtr.write<4>(*rseg_hdr, hist, mach_read_from_4(hist) - seg_size);
-
-	ut_ad(rseg->curr_size >= seg_size);
-
-	rseg->curr_size -= seg_size;
-
-	mutex_exit(&(rseg->mutex));
-
-	mtr_commit(&mtr);
+  while (!fseg_free_step(TRX_UNDO_SEG_HDR + TRX_UNDO_FSEG_HEADER +
+                         block->frame, &mtr));
 }
 
 /** Remove unnecessary history data from a rollback segment.
 @param[in,out]	rseg		rollback segment
-@param[in]	limit		truncate anything before this */
+@param[in]	limit		truncate anything before this
+@param[in]	all		whether everything can be truncated */
 static
 void
 trx_purge_truncate_rseg_history(
 	trx_rseg_t&			rseg,
-	const purge_sys_t::iterator&	limit)
+	const purge_sys_t::iterator&	limit,
+	bool				all)
 {
 	fil_addr_t	hdr_addr;
 	fil_addr_t	prev_hdr_addr;
@@ -431,8 +407,6 @@ trx_purge_truncate_rseg_history(
 	trx_id_t	undo_trx_no;
 
 	mtr.start();
-	ut_ad(rseg.is_persistent());
-	mutex_enter(&rseg.mutex);
 
 	buf_block_t* rseg_hdr = trx_rsegf_get(rseg.space, rseg.page_no, &mtr);
 
@@ -444,7 +418,6 @@ trx_purge_truncate_rseg_history(
 loop:
 	if (hdr_addr.page == FIL_NULL) {
 func_exit:
-		mutex_exit(&rseg.mutex);
 		mtr.commit();
 		return;
 	}
@@ -465,6 +438,10 @@ func_exit:
 		goto func_exit;
 	}
 
+	if (!all) {
+		goto func_exit;
+	}
+
 	prev_hdr_addr = flst_get_prev_addr(block->frame + hdr_addr.boffset
 					   + TRX_UNDO_HISTORY_NODE);
 	prev_hdr_addr.boffset = static_cast<uint16_t>(prev_hdr_addr.boffset
@@ -474,26 +451,17 @@ func_exit:
 	    == TRX_UNDO_TO_PURGE
 	    && !mach_read_from_2(block->frame + hdr_addr.boffset
 				 + TRX_UNDO_NEXT_LOG)) {
-
-		/* We can free the whole log segment */
-
-		mutex_exit(&rseg.mutex);
-		mtr.commit();
-
-		/* calls the trx_purge_remove_log_hdr()
-		inside trx_purge_free_segment(). */
-		trx_purge_free_segment(&rseg, hdr_addr);
+		/* We can free the whole log segment.
+		This will call trx_purge_remove_log_hdr(). */
+		trx_purge_free_segment(mtr, &rseg, hdr_addr);
 	} else {
 		/* Remove the log hdr from the rseg history. */
 		trx_purge_remove_log_hdr(rseg_hdr, block, hdr_addr.boffset,
 					 &mtr);
-
-		mutex_exit(&rseg.mutex);
-		mtr.commit();
 	}
 
+	mtr.commit();
 	mtr.start();
-	mutex_enter(&rseg.mutex);
 
 	rseg_hdr = trx_rsegf_get(rseg.space, rseg.page_no, &mtr);
 
@@ -547,10 +515,11 @@ __attribute__((optimize(0)))
 # endif
 #endif
 /**
-Removes unnecessary history data from rollback segments. NOTE that when this
-function is called, the caller must not have any latches on undo log pages!
+Remove unnecessary history data from rollback segments. NOTE that when this
+function is called, the caller (purge_coordinator_callback)
+must not have any latches on undo log pages!
 */
-static void trx_purge_truncate_history()
+void trx_purge_truncate_history()
 {
   ut_ad(purge_sys.head <= purge_sys.tail);
   purge_sys_t::iterator &head= purge_sys.head.trx_no
@@ -568,7 +537,13 @@ static void trx_purge_truncate_history()
     if (trx_rseg_t *rseg= trx_sys.rseg_array[i])
     {
       ut_ad(rseg->id == i);
-      trx_purge_truncate_rseg_history(*rseg, head);
+      ut_ad(rseg->is_persistent());
+      log_free_check();
+      mutex_enter(&rseg->mutex);
+      trx_purge_truncate_rseg_history(*rseg, head,
+                                      !rseg->trx_ref_count &&
+                                      purge_sys.sees(rseg->needs_purge));
+      mutex_exit(&rseg->mutex);
     }
   }
 
@@ -611,48 +586,45 @@ static void trx_purge_truncate_history()
 
     DBUG_LOG("undo", "marking for truncate: " << file->name);
 
-    for (ulint i= 0; i < TRX_SYS_N_RSEGS; ++i)
-      if (trx_rseg_t *rseg= trx_sys.rseg_array[i])
-        if (rseg->space == &space)
-          /* Once set, this rseg will not be allocated to subsequent
-          transactions, but we will wait for existing active
-          transactions to finish. */
-          rseg->skip_allocation= true;
-
     for (ulint i= 0; i < TRX_SYS_N_RSEGS; ++i)
     {
       trx_rseg_t *rseg= trx_sys.rseg_array[i];
       if (!rseg || rseg->space != &space)
         continue;
-      mutex_enter(&rseg->mutex);
-      ut_ad(rseg->skip_allocation);
       ut_ad(rseg->is_persistent());
-      if (rseg->trx_ref_count)
+
+      mutex_enter(&rseg->mutex);
+      /* Once set, this rseg will not be allocated to subsequent
+      transactions, but we will wait for existing active
+      transactions to finish and to be purged. */
+      rseg->skip_allocation = true;
+
+      if (rseg->trx_ref_count || !purge_sys.sees(rseg->needs_purge))
       {
-not_free:
+      not_free:
         mutex_exit(&rseg->mutex);
         return;
       }
 
-      if (rseg->curr_size != 1)
+      ut_ad(UT_LIST_GET_LEN(rseg->undo_list) == 0);
+      /* Check if all segments are cached and safe to remove. */
+      ulint cached= 0;
+
+      for (const trx_undo_t *undo= UT_LIST_GET_FIRST(rseg->undo_cached); undo;
+           undo= UT_LIST_GET_NEXT(undo_list, undo))
       {
-        /* Check if all segments are cached and safe to remove. */
-        ulint cached= 0;
-        for (trx_undo_t *undo= UT_LIST_GET_FIRST(rseg->undo_cached); undo;
-             undo= UT_LIST_GET_NEXT(undo_list, undo))
-        {
-          if (head.trx_no < undo->trx_id)
-            goto not_free;
-          else
-            cached+= undo->size;
-        }
-
-        ut_ad(rseg->curr_size > cached);
-
-        if (rseg->curr_size > cached + 1)
+        if (head.trx_no && head.trx_no < undo->trx_id)
           goto not_free;
+        else
+          cached+= undo->size;
       }
 
+      ut_ad(rseg->curr_size > cached);
+
+      if (rseg->curr_size > cached + 1 &&
+          (srv_fast_shutdown || srv_undo_sources || trx_sys.rseg_history_len))
+        goto not_free;
+
       mutex_exit(&rseg->mutex);
     }
 
@@ -673,6 +645,16 @@ not_free:
     mini-transaction commit and the server was killed, then
     discarding the to-be-trimmed pages without flushing would
     break crash recovery. */
+
+  rescan:
+    if (UNIV_UNLIKELY(srv_shutdown_state != SRV_SHUTDOWN_NONE) &&
+        srv_fast_shutdown)
+    {
+    fast_shutdown:
+      mtr.commit();
+      return;
+    }
+
     mysql_mutex_lock(&buf_pool.flush_list_mutex);
 
     for (buf_page_t *bpage= UT_LIST_GET_LAST(buf_pool.flush_list); bpage; )
@@ -682,13 +664,12 @@ not_free:
 
       buf_page_t *prev= UT_LIST_GET_PREV(list, bpage);
 
-      if (bpage->id().space() == space.id &&
-          bpage->oldest_modification() != 1)
+      if (bpage->oldest_modification() > 2 &&
+          bpage->id().space() == space.id)
       {
         ut_ad(bpage->state() == BUF_BLOCK_FILE_PAGE);
         auto block= reinterpret_cast<buf_block_t*>(bpage);
-        block->fix();
-        ut_ad(rw_lock_s_lock_nowait(block->debug_latch, __FILE__, __LINE__));
+        buf_block_buf_fix_inc(block, __FILE__, __LINE__);
         buf_pool.flush_hp.set(prev);
         mysql_mutex_unlock(&buf_pool.flush_list_mutex);
 
@@ -699,22 +680,24 @@ not_free:
         mysql_mutex_lock(&buf_pool.flush_list_mutex);
         ut_ad(bpage->io_fix() == BUF_IO_NONE);
 
-        if (bpage->oldest_modification() > 1)
-        {
-          bpage->clear_oldest_modification(false);
+        if (bpage->oldest_modification() > 2 &&
+            !mtr.have_x_latch(*reinterpret_cast<buf_block_t*>(bpage)))
           mtr.memo_push(block, MTR_MEMO_PAGE_X_FIX);
-        }
         else
         {
+          buf_block_buf_fix_dec(block);
           rw_lock_x_unlock(&block->lock);
-          block->unfix();
         }
 
         if (prev != buf_pool.flush_hp.get())
         {
-          /* Rescan, because we may have lost the position. */
-          bpage= UT_LIST_GET_LAST(buf_pool.flush_list);
-          continue;
+          /* The functions buf_pool_t::release_freed_page() or
+          buf_do_flush_list_batch() may be right now holding
+          buf_pool.mutex and waiting to acquire
+          buf_pool.flush_list_mutex. Ensure that they can proceed,
+          to avoid extreme waits. */
+          mysql_mutex_unlock(&buf_pool.flush_list_mutex);
+          goto rescan;
         }
       }
 
@@ -723,6 +706,10 @@ not_free:
 
     mysql_mutex_unlock(&buf_pool.flush_list_mutex);
 
+    if (UNIV_UNLIKELY(srv_shutdown_state != SRV_SHUTDOWN_NONE) &&
+        srv_fast_shutdown)
+      goto fast_shutdown;
+
     /* Adjust the tablespace metadata. */
     if (!fil_truncate_prepare(space.id))
     {
@@ -753,6 +740,8 @@ not_free:
 
       ut_ad(rseg->id == i);
       ut_ad(rseg->is_persistent());
+      ut_ad(!rseg->trx_ref_count);
+      ut_ad(!head.trx_no || rseg->needs_purge <= head.trx_no);
       ut_d(const auto old_page= rseg->page_no);
 
       buf_block_t *rblock= trx_rseg_header_create(&space, i,
@@ -775,9 +764,6 @@ not_free:
         ut_free(undo);
       }
 
-      UT_LIST_INIT(rseg->undo_list, &trx_undo_t::undo_list);
-      UT_LIST_INIT(rseg->undo_cached, &trx_undo_t::undo_list);
-
       /* These were written by trx_rseg_header_create(). */
       ut_ad(!mach_read_from_4(TRX_RSEG + TRX_RSEG_FORMAT + rblock->frame));
       ut_ad(!mach_read_from_4(TRX_RSEG + TRX_RSEG_HISTORY_SIZE +
@@ -786,9 +772,10 @@ not_free:
       the rseg header */
       rseg->curr_size= 1;
       rseg->trx_ref_count= 0;
+      rseg->needs_purge= 0;
+      rseg->skip_allocation= false;
       rseg->last_page_no= FIL_NULL;
       rseg->last_commit_and_offset= 0;
-      rseg->needs_purge= false;
     }
 
     mtr.commit_shrink(space);
@@ -812,17 +799,6 @@ not_free:
                     log_buffer_flush_to_disk();
                     DBUG_SUICIDE(););
 
-    for (ulint i= 0; i < TRX_SYS_N_RSEGS; ++i)
-    {
-      if (trx_rseg_t *rseg= trx_sys.rseg_array[i])
-      {
-        ut_ad(rseg->id == i);
-        ut_ad(rseg->is_persistent());
-        if (rseg->space == &space)
-          rseg->skip_allocation= false;
-      }
-    }
-
     ib::info() << "Truncated " << file->name;
     purge_sys.truncate.last= purge_sys.truncate.current;
     ut_ad(&space == purge_sys.truncate.current);
@@ -891,7 +867,6 @@ static void trx_purge_rseg_get_next_history_log(
 
 	trx_no = mach_read_from_8(log_hdr + TRX_UNDO_TRX_NO);
 	ut_ad(mach_read_from_2(log_hdr + TRX_UNDO_NEEDS_PURGE) <= 1);
-	const byte needs_purge = log_hdr[TRX_UNDO_NEEDS_PURGE + 1];
 
 	mtr.commit();
 
@@ -899,7 +874,6 @@ static void trx_purge_rseg_get_next_history_log(
 
 	purge_sys.rseg->last_page_no = prev_log_addr.page;
 	purge_sys.rseg->set_last_commit(prev_log_addr.boffset, trx_no);
-	purge_sys.rseg->needs_purge = needs_purge != 0;
 
 	/* Purge can also produce events, however these are already ordered
 	in the rollback segment and any user generated event will be greater
@@ -1271,9 +1245,8 @@ static void trx_purge_wait_for_workers_to_complete()
 /**
 Run a purge batch.
 @param n_tasks   number of purge tasks to submit to the queue
-@param truncate  whether to truncate the history at the end of the batch
 @return number of undo log pages handled in the batch */
-ulint trx_purge(ulint n_tasks, bool truncate)
+ulint trx_purge(ulint n_tasks)
 {
 	que_thr_t*	thr = NULL;
 	ulint		n_pages_handled;
@@ -1307,10 +1280,6 @@ ulint trx_purge(ulint n_tasks, bool truncate)
 
 	trx_purge_wait_for_workers_to_complete();
 
-	if (truncate) {
-		trx_purge_truncate_history();
-	}
-
 	MONITOR_INC_VALUE(MONITOR_PURGE_INVOKED, 1);
 	MONITOR_INC_VALUE(MONITOR_PURGE_N_PAGE_HANDLED, n_pages_handled);
 
