@@ -29,12 +29,10 @@
 
 /* static variables */
 
-#undef MIN_SORT_MEMORY
 #undef DISK_BUFFER_SIZE
 
 #define MERGEBUFF 15
 #define MERGEBUFF2 31
-#define MIN_SORT_MEMORY (4096-MALLOC_OVERHEAD)
 #define DISK_BUFFER_SIZE (IO_SIZE*128)
 
 /* How many keys we can keep in memory */
@@ -145,11 +143,11 @@ int _ma_create_index_by_sort(MARIA_SORT_PARAM *info, my_bool no_messages,
   sort_keys= (uchar **) NULL; error= 1;
   maxbuffer=1;
 
-  memavl=MY_MAX(sortbuff_size,MIN_SORT_MEMORY);
+  memavl=MY_MAX(sortbuff_size,MARIA_MIN_SORT_MEMORY);
   records=	info->sort_info->max_records;
   sort_length=	info->key_length;
 
-  while (memavl >= MIN_SORT_MEMORY)
+  while (memavl >= MARIA_MIN_SORT_MEMORY)
   {
     /* Check if we can fit all keys into memory */
     if (((ulonglong) (records + 1) *
@@ -179,8 +177,7 @@ int _ma_create_index_by_sort(MARIA_SORT_PARAM *info, my_bool no_messages,
 	maxbuffer_org= maxbuffer;
 	if (memavl < sizeof(BUFFPEK) * maxbuffer ||
 	    (keys= (memavl-sizeof(BUFFPEK)*maxbuffer)/
-             (sort_length+sizeof(char*))) <= 1 ||
-            keys < maxbuffer)
+             (sort_length+sizeof(char*))) <= 1)
 	{
 	  _ma_check_print_error(info->sort_info->param,
                                 "aria_sort_buffer_size is too small. Current aria_sort_buffer_size: %llu  rows: %llu  sort_length: %u",
@@ -189,6 +186,15 @@ int _ma_create_index_by_sort(MARIA_SORT_PARAM *info, my_bool no_messages,
           my_errno= ENOMEM;
 	  goto err;
 	}
+        if (keys < maxbuffer)
+        {
+          /*
+            There must be sufficient memory for at least one key per BUFFPEK,
+            otherwise repair by sort/parallel repair cannot operate.
+          */
+          maxbuffer= (uint) keys;
+          break;
+        }
       }
       while ((maxbuffer= (uint) (records/(keys-1)+1)) != maxbuffer_org);
     }
@@ -208,10 +214,10 @@ int _ma_create_index_by_sort(MARIA_SORT_PARAM *info, my_bool no_messages,
 	break;
     }
     old_memavl=memavl;
-    if ((memavl=memavl/4*3) < MIN_SORT_MEMORY && old_memavl > MIN_SORT_MEMORY)
-      memavl=MIN_SORT_MEMORY;
+    if ((memavl=memavl/4*3) < MARIA_MIN_SORT_MEMORY && old_memavl > MARIA_MIN_SORT_MEMORY)
+      memavl=MARIA_MIN_SORT_MEMORY;
   }
-  if (memavl < MIN_SORT_MEMORY)
+  if (memavl < MARIA_MIN_SORT_MEMORY)
   {
     /* purecov: begin inspected */
     _ma_check_print_error(info->sort_info->param,
@@ -233,6 +239,20 @@ int _ma_create_index_by_sort(MARIA_SORT_PARAM *info, my_bool no_messages,
       == HA_POS_ERROR)
     goto err; /* purecov: tested */
 
+  if (maxbuffer >= keys)
+  {
+    /*
+      merge_many_buff will crash if maxbuffer > keys as then we cannot store in memory
+      the keys for each buffer.
+    */
+    keys= maxbuffer + 1;
+    if (!(sort_keys= ((uchar **)
+                      my_realloc(PSI_INSTRUMENT_ME, sort_keys,
+                                 (size_t) (keys*(sort_length+sizeof(char*))+
+                                           HA_FT_MAXBYTELEN), MYF(MY_FREE_ON_ERROR)))))
+      goto err;
+  }
+
   info->sort_info->param->stage++;                         /* Merge stage */
 
   if (maxbuffer == 0)
@@ -387,12 +407,12 @@ static my_bool _ma_thr_find_all_keys_exec(MARIA_SORT_PARAM* sort_param)
   bzero((char*) &sort_param->unique, sizeof(sort_param->unique));
 
   sortbuff_size= sort_param->sortbuff_size;
-  memavl=       MY_MAX(sortbuff_size, MIN_SORT_MEMORY);
+  memavl=       MY_MAX(sortbuff_size, MARIA_MIN_SORT_MEMORY);
   idx=          (ha_keys) sort_param->sort_info->max_records;
   sort_length=  sort_param->key_length;
   maxbuffer=    1;
 
-  while (memavl >= MIN_SORT_MEMORY)
+  while (memavl >= MARIA_MIN_SORT_MEMORY)
   {
     if ((my_off_t) (idx+1)*(sort_length+sizeof(char*)) <= (my_off_t) memavl)
       keys= idx+1;
@@ -442,11 +462,11 @@ static my_bool _ma_thr_find_all_keys_exec(MARIA_SORT_PARAM* sort_param)
         break;
     }
     old_memavl= memavl;
-    if ((memavl= memavl/4*3) < MIN_SORT_MEMORY &&
-        old_memavl > MIN_SORT_MEMORY)
-      memavl= MIN_SORT_MEMORY;
+    if ((memavl= memavl/4*3) < MARIA_MIN_SORT_MEMORY &&
+        old_memavl > MARIA_MIN_SORT_MEMORY)
+      memavl= MARIA_MIN_SORT_MEMORY;
   }
-  if (memavl < MIN_SORT_MEMORY)
+  if (memavl < MARIA_MIN_SORT_MEMORY)
   {
     /* purecov: begin inspected */
     _ma_check_print_error(sort_param->sort_info->param,
@@ -626,7 +646,7 @@ int _ma_thr_write_keys(MARIA_SORT_PARAM *sort_param)
       if (!mergebuf)
       {
         length=(size_t)param->sort_buffer_length;
-        while (length >= MIN_SORT_MEMORY)
+        while (length >= MARIA_MIN_SORT_MEMORY)
         {
           if ((mergebuf= my_malloc(PSI_INSTRUMENT_ME, (size_t) length, MYF(0))))
               break;
@@ -732,8 +752,8 @@ static int write_keys(MARIA_SORT_PARAM *info, register uchar **sort_keys,
   if (!buffpek)
     DBUG_RETURN(1);                             /* Out of memory */
 
-  my_qsort2((uchar*) sort_keys,(size_t) count, sizeof(uchar*),
-            (qsort2_cmp) info->key_cmp, info);
+  my_qsort2(sort_keys, count, sizeof(uchar*),
+            info->key_cmp, info);
   if (!my_b_inited(tempfile) &&
       open_cached_file(tempfile, my_tmpdir(info->tmpdir), "ST",
                        DISK_BUFFER_SIZE, info->sort_info->param->myf_rw))
@@ -778,8 +798,8 @@ static int write_keys_varlen(MARIA_SORT_PARAM *info,
   if (!buffpek)
     DBUG_RETURN(1);                             /* Out of memory */
 
-  my_qsort2((uchar*) sort_keys, (size_t) count, sizeof(uchar*),
-            (qsort2_cmp) info->key_cmp, info);
+  my_qsort2(sort_keys, count, sizeof(uchar*),
+            info->key_cmp, info);
   if (!my_b_inited(tempfile) &&
       open_cached_file(tempfile, my_tmpdir(info->tmpdir), "ST",
                        DISK_BUFFER_SIZE, info->sort_info->param->myf_rw))
@@ -821,8 +841,8 @@ static int write_index(MARIA_SORT_PARAM *info, register uchar **sort_keys,
 {
   DBUG_ENTER("write_index");
 
-  my_qsort2((uchar*) sort_keys,(size_t) count,sizeof(uchar*),
-            (qsort2_cmp) info->key_cmp,info);
+  my_qsort2(sort_keys, count,sizeof(uchar*),
+            info->key_cmp,info);
   while (count--)
   {
     if ((*info->key_write)(info, *sort_keys++))
@@ -1024,8 +1044,8 @@ merge_buffers(MARIA_SORT_PARAM *info, ha_keys keys, IO_CACHE *from_file,
   sort_length=info->key_length;
 
   if (init_queue(&queue,(uint) (Tb-Fb)+1,offsetof(BUFFPEK,key),0,
-                 (int (*)(void*, uchar *,uchar*)) info->key_cmp,
-                 (void*) info, 0, 0))
+                 info->key_cmp,
+                 info, 0, 0))
     DBUG_RETURN(1); /* purecov: inspected */
 
   for (buffpek= Fb ; buffpek <= Tb ; buffpek++)
