@@ -150,21 +150,21 @@ struct st_maria_share;
 struct st_maria_handler;                        /* For referense */
 struct st_maria_keydef;
 
-typedef struct st_maria_key                 /* Internal info about a key */
+struct st_maria_key                 /* Internal info about a key */
 {
   uchar *data;                              /* Data for key */
   struct st_maria_keydef *keyinfo;          /* Definition for key */
   uint data_length;                         /* Length of key data */
   uint ref_length;                          /* record ref + transid */
   uint32 flag;                               /* 0 or SEARCH_PART_KEY */
-} MARIA_KEY;
+};
 
-typedef struct st_maria_decode_tree     /* Decode huff-table */
+struct st_maria_decode_tree     /* Decode huff-table */
 {
   uint16 *table;
   uint quick_table_bits;
   uchar *intervalls;
-} MARIA_DECODE_TREE;
+};
 
 
 typedef struct s3_info S3_INFO;
@@ -332,6 +332,8 @@ void _ma_update_auto_increment_key(HA_CHECK *param, MARIA_HA *info,
 #define MARIA_MAX_TREE_LEVELS 32
 #define MARIA_MAX_RECORD_ON_STACK 16384
 
+#define MARIA_MIN_SORT_MEMORY (16384-MALLOC_OVERHEAD)
+
 /* maria_open() flag, specific for maria_pack */
 #define HA_OPEN_IGNORE_MOVED_STATE (1U << 30)
 
@@ -396,7 +398,7 @@ typedef struct st_maria_sort_param
   my_bool calc_checksum;                /* calculate table checksum */
   size_t rec_buff_size;
 
-  int (*key_cmp)(struct st_maria_sort_param *, const void *, const void *);
+  int (*key_cmp)(void *, const void *, const void *);
   int (*key_read)(struct st_maria_sort_param *, uchar *);
   int (*key_write)(struct st_maria_sort_param *, const uchar *);
   void (*lock_in_memory)(HA_CHECK *);
@@ -752,6 +754,11 @@ typedef struct st_maria_share
   ulong max_pack_length;
   ulong state_diff_length;
   uint rec_reflength;			/* rec_reflength in use now */
+  /*
+    Extra flag to use for my_malloc(); set to MY_THREAD_SPECIFIC for temporary
+    tables whose memory allocation should be accounted to the current THD.
+  */
+  uint malloc_flag;
   uint keypage_header;
   uint32 ftkeys;			/* Number of distinct full-text keys
 						   + 1 */
@@ -787,6 +794,7 @@ typedef struct st_maria_share
   my_bool changed,			/* If changed since lock */
     global_changed,			/* If changed since open */
     not_flushed;
+  my_bool no_status_updates;            /* Set to 1 if S3 readonly table */
   my_bool internal_table;               /* Internal tmp table */
   my_bool lock_key_trees;               /* If we have to lock trees on read */
   my_bool non_transactional_concurrent_insert;
@@ -803,6 +811,7 @@ typedef struct st_maria_share
   my_bool key_del_used;                         /* != 0 if key_del is locked */
   my_bool deleting;                     /* we are going to delete this table */
   my_bool redo_error_given;             /* Used during recovery */
+  my_bool silence_encryption_errors;    /* Used during recovery */
   THR_LOCK lock;
   void (*lock_restore_status)(void *);
   /**
@@ -1202,7 +1211,8 @@ struct ha_table_option_struct
 #define PACK_TYPE_SELECTED	1U	/* Bits in field->pack_type */
 #define PACK_TYPE_SPACE_FIELDS	2U
 #define PACK_TYPE_ZERO_FILL	4U
-#define MARIA_FOUND_WRONG_KEY 32768U	/* Impossible value from ha_key_cmp */
+
+#define MARIA_FOUND_WRONG_KEY INT_MAX32	/* Impossible value from ha_key_cmp */
 
 #define MARIA_BLOCK_SIZE(key_length,data_pointer,key_pointer,block_size)  (((((key_length)+(data_pointer)+(key_pointer))*4+(key_pointer)+2)/(block_size)+1)*(block_size))
 #define MARIA_MAX_KEYPTR_SIZE	5	/* For calculating block lengths */
@@ -1584,7 +1594,6 @@ typedef struct st_maria_block_info
 #define PAGE_BUFFER_INIT	MY_ALIGN_DOWN(1024L*1024L*256L-MALLOC_OVERHEAD, 8192)
 #define READ_BUFFER_INIT	MY_ALIGN_DOWN(1024L*256L-MALLOC_OVERHEAD, 1024)
 #define SORT_BUFFER_INIT	MY_ALIGN_DOWN(1024L*1024L*256L-MALLOC_OVERHEAD, 1024)
-#define MIN_SORT_BUFFER		4096U
 
 #define fast_ma_writeinfo(INFO) if (!(INFO)->s->tot_locks) (void) _ma_writeinfo((INFO),0)
 #define fast_ma_readinfo(INFO) ((INFO)->lock_type == F_UNLCK) && _ma_readinfo((INFO),F_RDLCK,1)
@@ -1766,3 +1775,5 @@ static inline void decrement_share_in_trans(MARIA_SHARE *share)
 }
 C_MODE_END
 #endif
+
+#define CRASH_IF_S3_TABLE(share) DBUG_ASSERT(!share->no_status_updates)
