@@ -175,8 +175,7 @@ int _create_index_by_sort(MI_SORT_PARAM *info,my_bool no_messages,
 	maxbuffer_org= maxbuffer;
 	if (memavl < sizeof(BUFFPEK) * maxbuffer ||
 	    (keys= (memavl-sizeof(BUFFPEK)*maxbuffer)/
-             (sort_length+sizeof(char*))) <= 1 ||
-            keys < maxbuffer)
+             (sort_length+sizeof(char*))) <= 1)
 	{
 	  mi_check_print_error(info->sort_info->param,
 			       "myisam_sort_buffer_size is too small. Current myisam_sort_buffer_size: %llu  rows: %llu  sort_length: %u",
@@ -185,6 +184,15 @@ int _create_index_by_sort(MI_SORT_PARAM *info,my_bool no_messages,
           my_errno= ENOMEM;
 	  goto err;
 	}
+        if (keys < maxbuffer)
+        {
+          /*
+            There must be sufficient memory for at least one key per BUFFPEK,
+            otherwise repair by sort/parallel repair cannot operate.
+          */
+          keys= maxbuffer;
+          break;
+        }
       }
       while ((maxbuffer= (uint) (records/(keys-1)+1)) != maxbuffer_org);
     }
@@ -228,6 +236,20 @@ int _create_index_by_sort(MI_SORT_PARAM *info,my_bool no_messages,
                               &tempfile,&tempfile_for_exceptions))
       == HA_POS_ERROR)
     goto err; /* purecov: tested */
+  if (maxbuffer >= keys)
+  {
+    /*
+      merge_many_buff will crash if maxbuffer >= keys as then we cannot store in memory
+      the keys for each buffer.
+    */
+    keys= maxbuffer + 1;
+    if (!(sort_keys= ((uchar **)
+                      my_realloc(PSI_INSTRUMENT_ME, sort_keys,
+                                 (size_t) (keys*(sort_length+sizeof(char*))+
+                                           HA_FT_MAXBYTELEN), MYF(MY_FREE_ON_ERROR)))))
+      goto err;
+  }
+
   if (maxbuffer == 0)
   {
     if (!no_messages)
@@ -506,6 +528,11 @@ pthread_handler_t thr_find_all_keys(void *arg)
 {
   MI_SORT_PARAM *sort_param= (MI_SORT_PARAM*) arg;
   my_bool error= FALSE;
+
+  MI_SORT_INFO *si= sort_param->sort_info;
+  if (si->param->init_fix_record)
+    si->param->init_fix_record(si->info->external_ref);
+
   /* If my_thread_init fails */
   if (my_thread_init() || thr_find_all_keys_exec(sort_param))
     error= TRUE;
@@ -703,8 +730,8 @@ static int write_keys(MI_SORT_PARAM *info, register uchar **sort_keys,
   if (!buffpek)
     DBUG_RETURN(1);                             /* Out of memory */
 
-  my_qsort2((uchar*) sort_keys,(size_t) count, sizeof(uchar*),
-            (qsort2_cmp) info->key_cmp, info);
+  my_qsort2(sort_keys, count, sizeof(uchar *),
+            info->key_cmp, info);
   if (!my_b_inited(tempfile) &&
       open_cached_file(tempfile, my_tmpdir(info->tmpdir), "ST",
                        DISK_BUFFER_SIZE, info->sort_info->param->myf_rw))
@@ -749,8 +776,8 @@ static int write_keys_varlen(MI_SORT_PARAM *info,
   if (!buffpek)
     DBUG_RETURN(1);                             /* Out of memory */
 
-  my_qsort2((uchar*) sort_keys, (size_t) count, sizeof(uchar*),
-            (qsort2_cmp) info->key_cmp, info);
+  my_qsort2(sort_keys, count, sizeof(uchar *),
+            info->key_cmp, info);
   if (!my_b_inited(tempfile) &&
       open_cached_file(tempfile, my_tmpdir(info->tmpdir), "ST",
                        DISK_BUFFER_SIZE, info->sort_info->param->myf_rw))
@@ -791,8 +818,8 @@ static int write_index(MI_SORT_PARAM *info, register uchar **sort_keys,
 {
   DBUG_ENTER("write_index");
 
-  my_qsort2((uchar*) sort_keys,(size_t) count,sizeof(uchar*),
-           (qsort2_cmp) info->key_cmp,info);
+  my_qsort2(sort_keys, count, sizeof(uchar *),
+            info->key_cmp, info);
   while (count--)
   {
     if ((*info->key_write)(info,*sort_keys++))
@@ -977,7 +1004,7 @@ merge_buffers(MI_SORT_PARAM *info, ha_keys keys, IO_CACHE *from_file,
   sort_length=info->key_length;
 
   if (init_queue(&queue,(uint) (Tb-Fb)+1,offsetof(BUFFPEK,key),0,
-                 (int (*)(void*, uchar *,uchar*)) info->key_cmp,
+                 info->key_cmp,
                  (void*) info, 0, 0))
     DBUG_RETURN(1); /* purecov: inspected */
 
