@@ -720,11 +720,12 @@ typedef size_t GetKeyLength_t;
 typedef uint GetKeyLength_t;
 #endif
 
-static byte * sphinx_get_key ( const byte * pSharePtr, GetKeyLength_t * pLength, my_bool )
+static const uchar *sphinx_get_key(const void *pSharePtr,
+                                   GetKeyLength_t *pLength, my_bool)
 {
-	CSphSEShare * pShare = (CSphSEShare *) pSharePtr;
-	*pLength = (size_t) pShare->m_iTableNameLen;
-	return (byte*) pShare->m_sTable;
+  const CSphSEShare *pShare= static_cast<const CSphSEShare *>(pSharePtr);
+  *pLength= pShare->m_iTableNameLen;
+  return reinterpret_cast<const uchar *>(pShare->m_sTable);
 }
 
 #if MYSQL_VERSION_ID<50100
@@ -3538,7 +3539,8 @@ CSphSEStats * sphinx_get_stats ( THD * thd, SHOW_VAR * out )
 	return 0;
 }
 
-int sphinx_showfunc_total ( THD * thd, SHOW_VAR * out, char * )
+static int sphinx_showfunc_total ( THD * thd, SHOW_VAR * out, void *,
+                                   system_status_var *, enum_var_type )
 {
 	CSphSEStats * pStats = sphinx_get_stats ( thd, out );
 	if ( pStats )
@@ -3549,7 +3551,8 @@ int sphinx_showfunc_total ( THD * thd, SHOW_VAR * out, char * )
 	return 0;
 }
 
-int sphinx_showfunc_total_found ( THD * thd, SHOW_VAR * out, char * )
+static int sphinx_showfunc_total_found ( THD * thd, SHOW_VAR * out, void *,
+                                         system_status_var *, enum_var_type )
 {
 	CSphSEStats * pStats = sphinx_get_stats ( thd, out );
 	if ( pStats )
@@ -3560,7 +3563,8 @@ int sphinx_showfunc_total_found ( THD * thd, SHOW_VAR * out, char * )
 	return 0;
 }
 
-int sphinx_showfunc_time ( THD * thd, SHOW_VAR * out, char * )
+static int sphinx_showfunc_time ( THD * thd, SHOW_VAR * out, void *,
+                                  system_status_var *, enum_var_type )
 {
 	CSphSEStats * pStats = sphinx_get_stats ( thd, out );
 	if ( pStats )
@@ -3571,7 +3575,8 @@ int sphinx_showfunc_time ( THD * thd, SHOW_VAR * out, char * )
 	return 0;
 }
 
-int sphinx_showfunc_word_count ( THD * thd, SHOW_VAR * out, char * )
+static int sphinx_showfunc_word_count ( THD * thd, SHOW_VAR * out, void *,
+                                        system_status_var *, enum_var_type )
 {
 	CSphSEStats * pStats = sphinx_get_stats ( thd, out );
 	if ( pStats )
@@ -3582,9 +3587,11 @@ int sphinx_showfunc_word_count ( THD * thd, SHOW_VAR * out, char * )
 	return 0;
 }
 
-int sphinx_showfunc_words ( THD * thd, SHOW_VAR * out, char * sBuffer )
+static int sphinx_showfunc_words ( THD * thd, SHOW_VAR * out, void * buf,
+                                   system_status_var *, enum_var_type )
 {
 #if MYSQL_VERSION_ID>50100
+	char *sBuffer = static_cast<char*>(buf);
 	if ( sphinx_hton_ptr )
 	{
 		CSphTLS * pTls = (CSphTLS *) thd_get_ha_data ( thd, sphinx_hton_ptr );
@@ -3639,7 +3646,8 @@ int sphinx_showfunc_words ( THD * thd, SHOW_VAR * out, char * sBuffer )
 	return 0;
 }
 
-int sphinx_showfunc_error ( THD * thd, SHOW_VAR * out, char * )
+static int sphinx_showfunc_error ( THD * thd, SHOW_VAR * out, void *,
+                                   system_status_var *, enum_var_type )
 {
 	CSphSEStats * pStats = sphinx_get_stats ( thd, out );
 	out->type = SHOW_CHAR;
