@@ -37,7 +37,6 @@
 #define MYSQL_SERVER 1
 #include <my_global.h>
 #include "mysql_version.h"
-#include "spd_environ.h"
 #include "sql_priv.h"
 #include "probes_mysql.h"
 #include "sql_class.h"
@@ -109,7 +108,6 @@ extern volatile ulonglong spider_mon_table_cache_version_req;
       MYSQL_SYSVAR_NAME(param_name).def_val;                            \
   }
 
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
 static int spider_direct_update(THD *thd, SHOW_VAR *var, void *,
                                 system_status_var *, enum_var_type)
 {
@@ -133,7 +131,6 @@ static int spider_direct_delete(THD *thd, SHOW_VAR *var, void *,
     var->value = (char *) &trx->direct_delete_count;
   DBUG_RETURN(error_num);
 }
-#endif
 
 static int spider_direct_order_limit(THD *thd, SHOW_VAR *var, void *,
                                      system_status_var *, enum_var_type)
@@ -171,19 +168,6 @@ static int spider_parallel_search(THD *thd, SHOW_VAR *var, void *,
   DBUG_RETURN(error_num);
 }
 
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-static int spider_hs_result_free(THD *thd, SHOW_VAR *var, void *,
-                                 system_status_var *, enum_var_type)
-{
-  int error_num = 0;
-  SPIDER_TRX *trx;
-  DBUG_ENTER("spider_hs_result_free");
-  var->type = SHOW_LONGLONG;
-  if ((trx = spider_get_trx(thd, TRUE, &error_num)))
-    var->value = (char *) &trx->hs_result_free_count;
-  DBUG_RETURN(error_num);
-}
-#endif
 
 struct st_mysql_show_var spider_status_variables[] =
 {
@@ -191,7 +175,6 @@ struct st_mysql_show_var spider_status_variables[] =
     (char *) &spider_mon_table_cache_version, SHOW_LONGLONG},
   {"Spider_mon_table_cache_version_req",
     (char *) &spider_mon_table_cache_version_req, SHOW_LONGLONG},
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
 #ifdef SPIDER_HAS_SHOW_SIMPLE_FUNC
   {"Spider_direct_update", (char *) &spider_direct_update, SHOW_SIMPLE_FUNC},
   {"Spider_direct_delete", (char *) &spider_direct_delete, SHOW_SIMPLE_FUNC},
@@ -199,7 +182,6 @@ struct st_mysql_show_var spider_status_variables[] =
   {"Spider_direct_update", (char *) &spider_direct_update, SHOW_FUNC},
   {"Spider_direct_delete", (char *) &spider_direct_delete, SHOW_FUNC},
 #endif
-#endif
 #ifdef SPIDER_HAS_SHOW_SIMPLE_FUNC
   {"Spider_direct_order_limit",
     (char *) &spider_direct_order_limit, SHOW_SIMPLE_FUNC},
@@ -214,13 +196,6 @@ struct st_mysql_show_var spider_status_variables[] =
     (char *) &spider_direct_aggregate, SHOW_FUNC},
   {"Spider_parallel_search",
     (char *) &spider_parallel_search, SHOW_FUNC},
-#endif
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-#ifdef SPIDER_HAS_SHOW_SIMPLE_FUNC
-  {"Spider_hs_result_free", (char *) &spider_hs_result_free, SHOW_SIMPLE_FUNC},
-#else
-  {"Spider_hs_result_free", (char *) &spider_hs_result_free, SHOW_FUNC},
-#endif
 #endif
   {NullS, NullS, SHOW_LONG}
 };
@@ -582,25 +557,6 @@ static MYSQL_THDVAR_INT(
 
 SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, reset_sql_alloc)
 
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-/*
- -1 :fallback to default
-  0-:result free size for handlersocket
- */
-static MYSQL_THDVAR_LONGLONG(
-  hs_result_free_size, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Result free size for handlersocket", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  1048576, /* def */
-  -1, /* min */
-  9223372036854775807LL, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_OVERRIDE_VALUE_FUN(longlong, hs_result_free_size)
-#endif
 
 /*
  -1 :fallback to default
@@ -1536,25 +1492,6 @@ static MYSQL_THDVAR_INT(
 
 SPIDER_THDVAR_VALUE_FUNC(double, ping_interval_at_trx_start)
 
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-/*
-  0 :always ping
-  1-:interval
- */
-static MYSQL_THDVAR_INT(
-  hs_ping_interval, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Ping interval for handlersocket", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  30, /* def */
-  0, /* min */
-  2147483647, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_VALUE_FUNC(double, hs_ping_interval)
-#endif
 
 /*
  -1 :fallback to default
@@ -1866,7 +1803,7 @@ static MYSQL_THDVAR_INT(
   "Connect retry count", /* comment */
   NULL, /* check */
   NULL, /* update */
-  1000, /* def */
+  2, /* def */
   0, /* min */
   2147483647, /* max */
   0 /* blk */
@@ -1960,119 +1897,6 @@ static MYSQL_SYSVAR_LONGLONG(
 
 SPIDER_SYSVAR_OVERRIDE_VALUE_FUNC(longlong, udf_ct_bulk_insert_rows)
 
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-/*
-  0: no recycle
-  1: recycle in instance
-  2: recycle in thread
- */
-static MYSQL_THDVAR_UINT(
-  hs_r_conn_recycle_mode, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Handlersocket connection recycle mode", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  2, /* def */
-  0, /* min */
-  2, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_VALUE_FUNC(uint, hs_r_conn_recycle_mode)
-
-/*
-  0: weak
-  1: strict
- */
-static MYSQL_THDVAR_UINT(
-  hs_r_conn_recycle_strict, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Strict handlersocket connection recycle", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  0, /* def */
-  0, /* min */
-  1, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_VALUE_FUNC(uint, hs_r_conn_recycle_strict)
-
-/*
-  0: no recycle
-  1: recycle in instance
-  2: recycle in thread
- */
-static MYSQL_THDVAR_UINT(
-  hs_w_conn_recycle_mode, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Handlersocket connection recycle mode", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  2, /* def */
-  0, /* min */
-  2, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_VALUE_FUNC(uint, hs_w_conn_recycle_mode)
-
-/*
-  0: weak
-  1: strict
- */
-static MYSQL_THDVAR_UINT(
-  hs_w_conn_recycle_strict, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Strict handlersocket connection recycle", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  0, /* def */
-  0, /* min */
-  1, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_VALUE_FUNC(uint, hs_w_conn_recycle_strict)
-
-/*
- -1 :fallback to default
-  0 :not use
-  1 :use handlersocket
- */
-static MYSQL_THDVAR_INT(
-  use_hs_read, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Use handlersocket for reading", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  0, /* def */
-  -1, /* min */
-  1, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_OVERRIDE_VALUE_FUN(int, use_hs_read)
-
-/*
- -1 :fallback to default
-  0 :not use
-  1 :use handlersocket
- */
-static MYSQL_THDVAR_INT(
-  use_hs_write, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Use handlersocket for writing", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  0, /* def */
-  -1, /* min */
-  1, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_OVERRIDE_VALUE_FUN(int, use_hs_write)
-#endif
 
 /*
  -1 :fallback to default
@@ -2209,28 +2033,6 @@ static MYSQL_THDVAR_INT(
 
 SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, read_only_mode)
 
-#ifdef HA_CAN_BULK_ACCESS
-static int spider_bulk_access_free;
-/*
- -1 :fallback to default
-  0 :in reset
-  1 :in close
- */
-static MYSQL_SYSVAR_INT(
-  bulk_access_free,
-  spider_bulk_access_free,
-  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_READONLY,
-  "Free mode of bulk access resources",
-  NULL,
-  NULL,
-  0,
-  -1,
-  1,
-  0
-);
-
-SPIDER_SYSVAR_OVERRIDE_VALUE_FUN(int, bulk_access_free)
-#endif
 
 /*
  -1 :fallback to default
@@ -2744,9 +2546,6 @@ static struct st_mysql_sys_var* spider_system_variables[] = {
   MYSQL_SYSVAR(semi_split_read_limit),
   MYSQL_SYSVAR(init_sql_alloc_size),
   MYSQL_SYSVAR(reset_sql_alloc),
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  MYSQL_SYSVAR(hs_result_free_size),
-#endif
   MYSQL_SYSVAR(multi_split_read),
   MYSQL_SYSVAR(max_order),
   MYSQL_SYSVAR(semi_trx_isolation),
@@ -2807,9 +2606,6 @@ static struct st_mysql_sys_var* spider_system_variables[] = {
   MYSQL_SYSVAR(sts_bg_mode),
 #endif
   MYSQL_SYSVAR(ping_interval_at_trx_start),
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  MYSQL_SYSVAR(hs_ping_interval),
-#endif
   MYSQL_SYSVAR(auto_increment_mode),
   MYSQL_SYSVAR(same_server_link),
   MYSQL_SYSVAR(local_lock_table),
@@ -2832,14 +2628,6 @@ static struct st_mysql_sys_var* spider_system_variables[] = {
   MYSQL_SYSVAR(bka_mode),
   MYSQL_SYSVAR(udf_ct_bulk_insert_interval),
   MYSQL_SYSVAR(udf_ct_bulk_insert_rows),
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  MYSQL_SYSVAR(hs_r_conn_recycle_mode),
-  MYSQL_SYSVAR(hs_r_conn_recycle_strict),
-  MYSQL_SYSVAR(hs_w_conn_recycle_mode),
-  MYSQL_SYSVAR(hs_w_conn_recycle_strict),
-  MYSQL_SYSVAR(use_hs_read),
-  MYSQL_SYSVAR(use_hs_write),
-#endif
   MYSQL_SYSVAR(use_handler),
   MYSQL_SYSVAR(error_read_mode),
   MYSQL_SYSVAR(error_write_mode),
@@ -2847,9 +2635,6 @@ static struct st_mysql_sys_var* spider_system_variables[] = {
   MYSQL_SYSVAR(skip_parallel_search),
   MYSQL_SYSVAR(direct_order_limit),
   MYSQL_SYSVAR(read_only_mode),
-#ifdef HA_CAN_BULK_ACCESS
-  MYSQL_SYSVAR(bulk_access_free),
-#endif
   MYSQL_SYSVAR(udf_ds_use_real_table),
   MYSQL_SYSVAR(general_log),
   MYSQL_SYSVAR(index_hint_pushdown),
