@@ -27,6 +27,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 - 1301 USA*/
 #define NOMINMAX
 #endif
 #include <windows.h>
+#include <cassert>
+
 /**
   Windows-specific native file handle struct.
   Apart from the actual handle, contains PTP_IO
@@ -156,7 +158,7 @@ class aio
 {
 public:
   /**
-    Submit asyncronous IO.
+    Submit asynchronous IO.
     On completion, cb->m_callback is executed.
   */
   virtual int submit_io(aiocb *cb)= 0;
@@ -165,6 +167,10 @@ public:
   /** "Unind" file to AIO handler (used on Windows only) */
   virtual int unbind(const native_file_handle &fd)= 0;
   virtual ~aio(){};
+protected:
+  static void synchronous(aiocb *cb);
+  /** finish a partial read/write callback synchronously */
+  static void finish_synchronous(aiocb *cb);
 };
 
 class timer
@@ -198,21 +204,24 @@ protected:
   std::unique_ptr<aio> m_aio;
   virtual aio *create_native_aio(int max_io)= 0;
 
+public:
   /**
     Functions to be called at worker thread start/end
     can be used for example to set some TLS variables
   */
-  void (*m_worker_init_callback)(void);
-  void (*m_worker_destroy_callback)(void);
+  void (*m_worker_init_callback)(void)= [] {};
+  void (*m_worker_destroy_callback)(void)= [] {};
 
-public:
-  thread_pool() : m_aio(), m_worker_init_callback(), m_worker_destroy_callback()
+  thread_pool()
+      : m_aio()
   {
   }
   virtual void submit_task(task *t)= 0;
   virtual timer* create_timer(callback_func func, void *data=nullptr) = 0;
   void set_thread_callbacks(void (*init)(), void (*destroy)())
   {
+    assert(init);
+    assert(destroy);
     m_worker_init_callback= init;
     m_worker_destroy_callback= destroy;
   }
@@ -252,9 +261,9 @@ create_thread_pool_win(int min_threads= DEFAULT_MIN_POOL_THREADS,
   opened with FILE_FLAG_OVERLAPPED, and bound to completion
   port.
 */
-int pwrite(const native_file_handle &h, void *buf, size_t count,
+SSIZE_T pwrite(const native_file_handle &h, void *buf, size_t count,
            unsigned long long offset);
-int pread(const native_file_handle &h, void *buf, size_t count,
+SSIZE_T pread(const native_file_handle &h, void *buf, size_t count,
           unsigned long long offset);
 HANDLE win_get_syncio_event();
 #endif
